@@ -486,6 +486,9 @@ buildOpenAL() {
       -D CMAKE_INSTALL_PREFIX=$OPENAL_PATH \
       -D CMAKE_BUILD_TYPE=Release \
       -D LIBTYPE=STATIC \
+      -D ALSOFT_EXAMPLES=OFF \
+      -D ALSOFT_TESTS=OFF \
+      -D ALSOFT_UTILS=OFF \
       ..
   make $MAKE_ARGS
   sudo make install
@@ -608,7 +611,7 @@ buildCustomQt() {
   ./configure -prefix $QT_PATH -release -opensource -confirm-license -qt-zlib \
               -qt-libpng -qt-libjpeg -qt-freetype -qt-harfbuzz -qt-pcre -qt-xcb \
               -qt-xkbcommon-x11 -no-opengl -no-gtkstyle -static \
-              -nomake examples -nomake tests \
+              -nomake examples -nomake tests -no-mirclient \
               -dbus-runtime -no-gstreamer -no-mtdev # <- Not sure about these
   make $MAKE_ARGS
   sudo make install
