@@ -278,10 +278,10 @@ AuthSession::AuthSession(UserId userId)
 , _changelogs(Core::Changelogs::Create(this)) {
 	Expects(_userId != 0);
 
-	_saveDataTimer.setCallback([this] {
+	_saveDataTimer.setCallback([=] {
 		Local::writeUserSettings();
 	});
-	subscribe(Messenger::Instance().passcodedChanged(), [this] {
+	subscribe(Messenger::Instance().passcodedChanged(), [=] {
 		_shouldLockAt = 0;
 		notifications().updateAll();
 	});
