@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "boxes/peers/edit_peer_info_box.h"
 
@@ -50,6 +37,8 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 namespace {
 
 constexpr auto kUsernameCheckTimeout = TimeMs(200);
+constexpr auto kMinUsernameLength = 5;
+constexpr auto kMaxChannelDescription = 255; // See also add_contact_box.
 
 class Controller
 	: private MTP::Sender
@@ -336,6 +325,7 @@ object_ptr<Ui::RpWidget> Controller::createDescriptionEdit() {
 			langFactory(lng_create_group_description),
 			channel->about()),
 		st::editPeerDescriptionMargins);
+	result->entity()->setMaxLength(kMaxChannelDescription);
 
 	QObject::connect(
 		result->entity(),
@@ -516,7 +506,7 @@ void Controller::checkUsernameAvailability() {
 	auto checking = initial
 		? qsl(".bad.")
 		: _controls.username->getLastText().trimmed();
-	if (checking.size() < MinUsernameLength) {
+	if (checking.size() < kMinUsernameLength) {
 		return;
 	}
 	if (_checkUsernameRequestId) {
@@ -593,7 +583,7 @@ void Controller::usernameChanged() {
 	if (bad) {
 		showUsernameError(
 			Lang::Viewer(lng_create_channel_link_bad_symbols));
-	} else if (username.size() < MinUsernameLength) {
+	} else if (username.size() < kMinUsernameLength) {
 		showUsernameError(
 			Lang::Viewer(lng_create_channel_link_too_short));
 	} else {
@@ -1264,11 +1254,9 @@ void Controller::saveTitle() {
 			continueSave();
 			return;
 		}
+		_controls.title->showError();
 		if (type == qstr("NO_CHAT_TITLE")) {
-			_controls.title->showError();
 			_box->scrollToWidget(_controls.title);
-		} else {
-			_controls.title->setFocus();
 		}
 		cancelSave();
 	};
@@ -1314,7 +1302,7 @@ void Controller::saveDescription() {
 			successCallback();
 			return;
 		}
-		_controls.description->setFocus();
+		_controls.description->showError();
 		cancelSave();
 	}).send();
 }
