@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "boxes/send_files_box.h"
 
@@ -37,6 +24,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "window/window_controller.h"
 #include "styles/style_history.h"
 #include "styles/style_boxes.h"
+#include "layout.h"
 
 namespace {
 
@@ -796,7 +784,11 @@ void SingleFilePreview::preparePreview(const Storage::PreparedFile &file) {
 	prepareThumb(preview);
 	const auto filepath = file.path;
 	if (filepath.isEmpty()) {
-		auto filename = filedialogDefaultName(qsl("image"), qsl(".png"), QString(), true);
+		auto filename = filedialogDefaultName(
+			qsl("image"),
+			qsl(".png"),
+			QString(),
+			true);
 		_nameText.setText(
 			st::semiboldTextStyle,
 			filename,
@@ -1446,7 +1438,7 @@ void SendFilesBox::initSendWay() {
 				? SendFilesWay::Album
 				: SendFilesWay::Photos;
 		}
-		const auto currentWay = Auth().data().sendFilesWay();
+		const auto currentWay = Auth().settings().sendFilesWay();
 		if (currentWay == SendFilesWay::Files) {
 			return currentWay;
 		} else if (currentWay == SendFilesWay::Album) {
@@ -1589,18 +1581,21 @@ void SendFilesBox::captionResized() {
 	update();
 }
 
+bool SendFilesBox::canAddUrls(const QList<QUrl> &urls) const {
+	return !urls.isEmpty() && ranges::find_if(
+		urls,
+		[](const QUrl &url) { return !url.isLocalFile(); }
+	) == urls.end();
+}
+
 bool SendFilesBox::canAddFiles(not_null<const QMimeData*> data) const {
-	auto files = 0;
-	if (data->hasUrls()) {
-		for (const auto &url : data->urls()) {
-			if (url.isLocalFile()) {
-				++files;
-			}
-		}
-	} else if (data->hasImage()) {
-		++files;
+	const auto urls = data->hasUrls() ? data->urls() : QList<QUrl>();
+	auto filesCount = canAddUrls(urls) ? urls.size() : 0;
+	if (!filesCount && data->hasImage()) {
+		++filesCount;
 	}
-	if (_list.files.size() + files > Storage::MaxAlbumItems()) {
+
+	if (_list.files.size() + filesCount > Storage::MaxAlbumItems()) {
 		return false;
 	} else if (_list.files.size() > 1 && !_albumPreview) {
 		return false;
@@ -1613,10 +1608,14 @@ bool SendFilesBox::canAddFiles(not_null<const QMimeData*> data) const {
 
 bool SendFilesBox::addFiles(not_null<const QMimeData*> data) {
 	auto list = [&] {
-		if (data->hasUrls()) {
-			return Storage::PrepareMediaList(
-				data->urls(),
-				st::sendMediaPreviewSize);
+		const auto urls = data->hasUrls() ? data->urls() : QList<QUrl>();
+		auto result = canAddUrls(urls)
+			? Storage::PrepareMediaList(urls, st::sendMediaPreviewSize)
+			: Storage::PreparedList(
+				Storage::PreparedList::Error::EmptyFile,
+				QString());
+		if (result.error == Storage::PreparedList::Error::None) {
+			return result;
 		} else if (data->hasImage()) {
 			auto image = qvariant_cast<QImage>(data->imageData());
 			if (!image.isNull()) {
@@ -1626,9 +1625,7 @@ bool SendFilesBox::addFiles(not_null<const QMimeData*> data) {
 					st::sendMediaPreviewSize);
 			}
 		}
-		return Storage::PreparedList(
-			Storage::PreparedList::Error::EmptyFile,
-			QString());
+		return result;
 	}();
 	if (_list.files.size() + list.files.size() > Storage::MaxAlbumItems()) {
 		return false;
@@ -1769,7 +1766,7 @@ void SendFilesBox::send(bool ctrlShiftEnter) {
 	const auto way = _sendWay ? _sendWay->value() : Way::Files;
 
 	if (_compressConfirm == CompressConfirm::Auto) {
-		const auto oldWay = Auth().data().sendFilesWay();
+		const auto oldWay = Auth().settings().sendFilesWay();
 		if (way != oldWay) {
 			// Check if the user _could_ use the old value, but didn't.
 			if ((oldWay == Way::Album && _sendAlbum)
@@ -1777,8 +1774,8 @@ void SendFilesBox::send(bool ctrlShiftEnter) {
 				|| (oldWay == Way::Files && _sendFiles)
 				|| (way == Way::Files && (_sendAlbum || _sendPhotos))) {
 				// And in that case save it to settings.
-				Auth().data().setSendFilesWay(way);
-				Auth().saveDataDelayed();
+				Auth().settings().setSendFilesWay(way);
+				Auth().saveSettingsDelayed();
 			}
 		}
 	}
