@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -63,7 +50,7 @@ public:
 	Inner(QWidget *parent, const MTPInputStickerSet &set);
 
 	bool loaded() const;
-	int32 notInstalled() const;
+	bool notInstalled() const;
 	bool official() const;
 	base::lambda<TextWithEntities()> title() const;
 	QString shortName() const;
@@ -111,9 +98,10 @@ private:
 	uint64 _setId = 0;
 	uint64 _setAccess = 0;
 	QString _setTitle, _setShortName;
-	int32 _setCount = 0;
+	int _setCount = 0;
 	int32 _setHash = 0;
 	MTPDstickerSet::Flags _setFlags = 0;
+	TimeId _setInstallDate = TimeId(0);
 
 	MTPInputStickerSet _input;
 
