@@ -1,27 +1,16 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
 #include "chat_helpers/tabbed_selector.h"
+#include "chat_helpers/stickers.h"
 #include "base/variant.h"
+#include "base/timer.h"
 
 namespace Window {
 class Controller;
@@ -29,6 +18,7 @@ class Controller;
 
 namespace Ui {
 class LinkButton;
+class RippleAnimation;
 } // namespace Ui
 
 namespace ChatHelpers {
@@ -42,7 +32,9 @@ class StickersListWidget
 	Q_OBJECT
 
 public:
-	StickersListWidget(QWidget *parent, not_null<Window::Controller*> controller);
+	StickersListWidget(
+		QWidget *parent,
+		not_null<Window::Controller*> controller);
 
 	void refreshRecent() override;
 	void preloadImages() override;
@@ -52,6 +44,9 @@ public:
 	void showStickerSet(uint64 setId);
 	void showMegagroupSet(ChannelData *megagroup);
 
+	void afterShown() override;
+	void beforeHiding() override;
+
 	void refreshStickers();
 
 	void fillIcons(QList<StickerIcon> &icons);
@@ -63,6 +58,9 @@ public:
 	void notInstalledLocally(uint64 setId);
 	void clearInstalledLocally();
 
+	void sendSearchRequest();
+	void searchForSets(const QString &query);
+
 	~StickersListWidget();
 
 protected:
@@ -99,6 +97,7 @@ private:
 	enum class Section {
 		Featured,
 		Stickers,
+		Search,
 	};
 
 	struct OverSticker {
@@ -142,17 +141,22 @@ private:
 			uint64 id,
 			MTPDstickerSet::Flags flags,
 			const QString &title,
-			int hoversSize,
+			const QString &shortName,
+			bool externalLayout,
+			int count,
 			const Stickers::Pack &pack = Stickers::Pack());
 		Set(Set &&other);
 		Set &operator=(Set &&other);
 		~Set();
 
-		uint64 id;
-		MTPDstickerSet::Flags flags;
+		uint64 id = 0;
+		MTPDstickerSet::Flags flags = MTPDstickerSet::Flags();
 		QString title;
+		QString shortName;
 		Stickers::Pack pack;
 		std::unique_ptr<Ui::RippleAnimation> ripple;
+		bool externalLayout = false;
+		int count = 0;
 	};
 
 	template <typename Callback>
@@ -164,9 +168,15 @@ private:
 	void installSet(uint64 setId);
 	void removeMegagroupSet(bool locally);
 	void removeSet(uint64 setId);
+	void sendInstallRequest(
+		uint64 setId,
+		const MTPInputStickerSet &input);
+	void refreshSearchSets();
+	void refreshSearchIndex();
 
 	bool setHasTitle(const Set &set) const;
 	bool stickerHasDeleteButton(const Set &set, int index) const;
+	Stickers::Pack collectRecentStickers();
 	void refreshRecentStickers(bool resize = true);
 	void refreshFavedStickers();
 	enum class GroupStickersPlace {
@@ -189,12 +199,8 @@ private:
 	};
 	void validateSelectedIcon(ValidateIconAnimations animations);
 
-	std::vector<Set> &shownSets() {
-		return (_section == Section::Featured) ? _featuredSets : _mySets;
-	}
-	const std::vector<Set> &shownSets() const {
-		return (_section == Section::Featured) ? _featuredSets : _mySets;
-	}
+	std::vector<Set> &shownSets();
+	const std::vector<Set> &shownSets() const;
 	int featuredRowHeight() const;
 	void readVisibleSets();
 
@@ -202,6 +208,7 @@ private:
 	void paintStickers(Painter &p, QRect clip);
 	void paintMegagroupEmptySet(Painter &p, int y, bool buttonSelected, TimeMs ms);
 	void paintSticker(Painter &p, Set &set, int y, int index, bool selected, bool deleteSelected);
+	void paintEmptySearchResults(Painter &p);
 
 	int stickersRight() const;
 	bool featuredHasAddButton(int index) const;
@@ -220,6 +227,7 @@ private:
 	void appendSet(
 		std::vector<Set> &to,
 		uint64 setId,
+		bool externalLayout,
 		AppendSkip skip = AppendSkip::None);
 
 	void selectEmoji(EmojiPtr emoji);
@@ -231,9 +239,19 @@ private:
 	void setColumnCount(int count);
 	void refreshFooterIcons();
 
+	void cancelSetsSearch();
+	void showSearchResults();
+	void searchResultsDone(const MTPmessages_FoundStickerSets &result);
+	void refreshSearchRows();
+	void refreshSearchRows(const std::vector<Stickers::Set*> *cloudSets);
+	void fillLocalSearchRows(const QString &query);
+	void fillCloudSearchRows(const std::vector<Stickers::Set*> &sets);
+	void addSearchRow(not_null<const Stickers::Set*> set);
+
 	ChannelData *_megagroupSet = nullptr;
 	std::vector<Set> _mySets;
 	std::vector<Set> _featuredSets;
+	std::vector<Set> _searchSets;
 	base::flat_set<uint64> _installedLocallySets;
 	std::vector<bool> _custom;
 	base::flat_set<not_null<DocumentData*>> _favedStickersMap;
@@ -266,6 +284,12 @@ private:
 	QTimer _previewTimer;
 	bool _previewShown = false;
 
+	std::map<QString, std::vector<Stickers::Set*>> _searchCache;
+	std::vector<std::pair<uint64, QStringList>> _searchIndex;
+	base::Timer _searchRequestTimer;
+	QString _searchQuery, _searchNextQuery;
+	mtpRequestId _searchRequestId = 0;
+
 };
 
 } // namespace ChatHelpers
