@@ -1,22 +1,9 @@
 '''
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 '''
 import glob, re, binascii, os, sys
 
@@ -910,23 +897,10 @@ WARNING! All changes made in this file will be lost!\n\
 Created from \'' + os.path.basename(input_file) + '\' by \'codegen_scheme\'\n\
 \n\
 This file is part of Telegram Desktop,\n\
-the official desktop version of Telegram messaging app, see https://telegram.org\n\
+the official desktop application for the Telegram messaging service.\n\
 \n\
-Telegram Desktop is free software: you can redistribute it and/or modify\n\
-it under the terms of the GNU General Public License as published by\n\
-the Free Software Foundation, either version 3 of the License, or\n\
-(at your option) any later version.\n\
-\n\
-It is distributed in the hope that it will be useful,\n\
-but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n\
-GNU General Public License for more details.\n\
-\n\
-In addition, as a special exception, the copyright holders give permission\n\
-to link the code of portions of this program with the OpenSSL library.\n\
-\n\
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE\n\
-Copyright (c) 2014 John Preston, https://desktop.telegram.org\n\
+For license and copyright information please follow this link:\n\
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL\n\
 */\n\
 #pragma once\n\
 \n\
@@ -974,20 +948,10 @@ WARNING! All changes made in this file will be lost!\n\
 Created from \'' + os.path.basename(input_file) + '\' by \'codegen_scheme\'\n\
 \n\
 This file is part of Telegram Desktop,\n\
-the official desktop version of Telegram messaging app, see https://telegram.org\n\
+the official desktop application for the Telegram messaging service.\n\
 \n\
-Telegram Desktop is free software: you can redistribute it and/or modify\n\
-it under the terms of the GNU General Public License as published by\n\
-the Free Software Foundation, either version 3 of the License, or\n\
-(at your option) any later version.\n\
-\n\
-It is distributed in the hope that it will be useful,\n\
-but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n\
-GNU General Public License for more details.\n\
-\n\
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE\n\
-Copyright (c) 2014 John Preston, https://desktop.telegram.org\n\
+For license and copyright information please follow this link:\n\
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL\n\
 */\n\
 #include "scheme.h"\n\
 \n\
