@@ -1,28 +1,17 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "core/click_handler_types.h"
 
 #include "lang/lang_keys.h"
 #include "messenger.h"
 #include "platform/platform_specific.h"
+#include "history/view/history_view_element.h"
+#include "history/history_item.h"
 #include "boxes/confirm_box.h"
 #include "base/qthelp_regex.h"
 #include "base/qthelp_url.h"
@@ -49,12 +38,16 @@ QString tryConvertUrlToLocal(QString url) {
 		} else if (auto confirmPhoneMatch = regex_match(qsl("^confirmphone/?\\?(.+)"), query, matchOptions)) {
 			return qsl("tg://confirmphone?") + confirmPhoneMatch->captured(1);
 		} else if (auto ivMatch = regex_match(qsl("iv/?\\?(.+)(#|$)"), query, matchOptions)) {
-			auto params = url_parse_params(ivMatch->captured(1), UrlParamNameTransform::ToLower);
-			auto previewedUrl = params.value(qsl("url"));
-			if (previewedUrl.startsWith(qstr("http://"), Qt::CaseInsensitive)
-				|| previewedUrl.startsWith(qstr("https://"), Qt::CaseInsensitive)) {
-				return previewedUrl;
-			}
+			//
+			// We need to show our t.me page, not the url directly.
+			//
+			//auto params = url_parse_params(ivMatch->captured(1), UrlParamNameTransform::ToLower);
+			//auto previewedUrl = params.value(qsl("url"));
+			//if (previewedUrl.startsWith(qstr("http://"), Qt::CaseInsensitive)
+			//	|| previewedUrl.startsWith(qstr("https://"), Qt::CaseInsensitive)) {
+			//	return previewedUrl;
+			//}
+			return url;
 		} else if (auto socksMatch = regex_match(qsl("socks/?\\?(.+)(#|$)"), query, matchOptions)) {
 			return qsl("tg://socks?") + socksMatch->captured(1);
 		} else if (auto usernameMatch = regex_match(qsl("^([a-zA-Z0-9\\.\\_]+)(/?\\?|/?$|/(\\d+)/?(?:\\?|$))"), query, matchOptions)) {
@@ -76,6 +69,20 @@ bool UrlRequiresConfirmation(const QUrl &url) {
 
 } // namespace
 
+UrlClickHandler::UrlClickHandler(const QString &url, bool fullDisplayed)
+: TextClickHandler(fullDisplayed)
+, _originalUrl(url) {
+	if (isEmail()) {
+		_readable = _originalUrl;
+	} else {
+		const auto original = QUrl(_originalUrl);
+		const auto good = QUrl(original.isValid()
+			? original.toEncoded()
+			: QString());
+		_readable = good.isValid() ? good.toDisplayString() : _originalUrl;
+	}
+}
+
 QString UrlClickHandler::copyToClipboardContextItemText() const {
 	return lang(isEmail() ? lng_context_copy_email : lng_context_copy_link);
 }
@@ -241,6 +248,23 @@ TextWithEntities HashtagClickHandler::getExpandedLinkTextWithEntities(ExpandLink
 	return simpleTextWithEntity({ EntityInTextHashtag, entityOffset, textPart.size() });
 }
 
+QString CashtagClickHandler::copyToClipboardContextItemText() const {
+	return lang(lng_context_copy_hashtag);
+}
+
+void CashtagClickHandler::onClick(Qt::MouseButton button) const {
+	if (button == Qt::LeftButton || button == Qt::MiddleButton) {
+		App::searchByHashtag(_tag, Ui::getPeerForMouseAction());
+	}
+}
+
+TextWithEntities CashtagClickHandler::getExpandedLinkTextWithEntities(
+		ExpandLinksMode mode,
+		int entityOffset,
+		const QStringRef &textPart) const {
+	return simpleTextWithEntity({ EntityInTextCashtag, entityOffset, textPart.size() });
+}
+
 PeerData *BotCommandClickHandler::_peer = nullptr;
 UserData *BotCommandClickHandler::_bot = nullptr;
 void BotCommandClickHandler::onClick(Qt::MouseButton button) const {
@@ -254,10 +278,11 @@ void BotCommandClickHandler::onClick(Qt::MouseButton button) const {
 		}
 
 		if (auto peer = Ui::getPeerForMouseAction()) { // old way
-			UserData *bot = peer->isUser() ? peer->asUser() : nullptr;
-			if (auto item = App::hoveredLinkItem()) {
-				if (!bot) {
-					bot = item->fromOriginal()->asUser(); // may return nullptr
+			auto bot = peer->isUser() ? peer->asUser() : nullptr;
+			if (!bot) {
+				if (const auto view = App::hoveredLinkItem()) {
+					// may return nullptr
+					bot = view->data()->fromOriginal()->asUser();
 				}
 			}
 			Ui::showPeerHistory(peer, ShowAtTheEndMsgId);
