@@ -1,27 +1,15 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "data/data_drafts.h"
 
 #include "ui/widgets/input_fields.h"
 #include "chat_helpers/message_field.h"
+#include "history/history.h"
 #include "history/history_widget.h"
 #include "mainwidget.h"
 #include "storage/localstorage.h"
@@ -31,11 +19,28 @@ namespace {
 
 } // namespace
 
-Draft::Draft(const Ui::FlatTextarea *field, MsgId msgId, bool previewCancelled, mtpRequestId saveRequestId)
-	: textWithTags(field->getTextWithTags())
-	, msgId(msgId)
-	, cursor(field)
-	, previewCancelled(previewCancelled) {
+Draft::Draft(
+	const TextWithTags &textWithTags,
+	MsgId msgId,
+	const MessageCursor &cursor,
+	bool previewCancelled,
+	mtpRequestId saveRequestId)
+: textWithTags(textWithTags)
+, msgId(msgId)
+, cursor(cursor)
+, previewCancelled(previewCancelled)
+, saveRequestId(saveRequestId) {
+}
+
+Draft::Draft(
+	not_null<const Ui::FlatTextarea*> field,
+	MsgId msgId,
+	bool previewCancelled,
+	mtpRequestId saveRequestId)
+: textWithTags(field->getTextWithTags())
+, msgId(msgId)
+, cursor(field)
+, previewCancelled(previewCancelled) {
 }
 
 void applyPeerCloudDraft(PeerId peerId, const MTPDdraftMessage &draft) {
@@ -44,7 +49,7 @@ void applyPeerCloudDraft(PeerId peerId, const MTPDdraftMessage &draft) {
 	auto textWithTags = TextWithTags { TextUtilities::ApplyEntities(text), ConvertEntitiesToTextTags(text.entities) };
 	auto replyTo = draft.has_reply_to_msg_id() ? draft.vreply_to_msg_id.v : MsgId(0);
 	auto cloudDraft = std::make_unique<Draft>(textWithTags, replyTo, MessageCursor(QFIXED_MAX, QFIXED_MAX, QFIXED_MAX), draft.is_no_webpage());
-	cloudDraft->date = ::date(draft.vdate);
+	cloudDraft->date = draft.vdate.v;
 
 	history->setCloudDraft(std::move(cloudDraft));
 	history->createLocalDraftFromCloud();
