@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "data/data_peer.h"
 
@@ -25,6 +12,9 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "data/data_peer_values.h"
 #include "data/data_channel_admins.h"
 #include "data/data_photo.h"
+#include "data/data_feed.h"
+#include "data/data_session.h"
+#include "history/history.h"
 #include "lang/lang_keys.h"
 #include "observer_peer.h"
 #include "mainwidget.h"
@@ -35,6 +25,7 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "messenger.h"
 #include "mainwindow.h"
 #include "window/window_controller.h"
+#include "storage/localstorage.h"
 #include "ui/empty_userpic.h"
 #include "ui/text_options.h"
 
@@ -82,7 +73,9 @@ PeerClickHandler::PeerClickHandler(not_null<PeerData*> peer)
 void PeerClickHandler::onClick(Qt::MouseButton button) const {
 	if (button == Qt::LeftButton && App::wnd()) {
 		auto controller = App::wnd()->controller();
-		if (_peer && _peer->isChannel() && controller->historyPeer.current() != _peer) {
+		if (_peer
+			&& _peer->isChannel()
+			&& controller->activeChatCurrent().peer() != _peer) {
 			if (!_peer->asChannel()->isPublic() && !_peer->asChannel()->amIn()) {
 				Ui::show(Box<InformBox>(lang(_peer->isMegagroup()
 					? lng_group_not_accessible
@@ -130,7 +123,7 @@ void PeerData::updateNameDelayed(
 
 	Notify::PeerUpdate update(this);
 	update.flags |= UpdateFlag::NameChanged;
-	update.oldNameFirstChars = nameFirstChars();
+	update.oldNameFirstLetters = nameFirstLetters();
 
 	if (isUser()) {
 		if (asUser()->username != newUsername) {
@@ -180,7 +173,7 @@ void PeerData::setUserpic(
 
 void PeerData::setUserpicPhoto(const MTPPhoto &data) {
 	auto photoId = [&]() -> PhotoId {
-		if (auto photo = App::feedPhoto(data)) {
+		if (const auto photo = Auth().data().photo(data)) {
 			photo->peer = this;
 			return photo->id;
 		}
@@ -308,12 +301,19 @@ void PeerData::setUserpicChecked(
 		|| _userpicLocation != location) {
 		setUserpic(photoId, location, userpic);
 		Notify::peerUpdatedDelayed(this, UpdateFlag::PhotoChanged);
+		if (const auto channel = asChannel()) {
+			if (const auto feed = channel->feed()) {
+				Auth().data().notifyFeedUpdated(
+					feed,
+					Data::FeedUpdateFlag::ChannelPhoto);
+			}
+		}
 	}
 }
 
 void PeerData::fillNames() {
 	_nameWords.clear();
-	_nameFirstChars.clear();
+	_nameFirstLetters.clear();
 	auto toIndexList = QStringList();
 	auto appendToIndex = [&](const QString &value) {
 		if (!value.isEmpty()) {
@@ -327,7 +327,7 @@ void PeerData::fillNames() {
 	if (appendTranslit) {
 		appendToIndex(translitRusEng(toIndexList.front()));
 	}
-	if (auto user = asUser()) {
+	if (const auto user = asUser()) {
 		if (user->nameOrPhone != name) {
 			appendToIndex(user->nameOrPhone);
 		}
@@ -335,7 +335,7 @@ void PeerData::fillNames() {
 		if (isSelf()) {
 			appendToIndex(lang(lng_saved_messages));
 		}
-	} else if (auto channel = asChannel()) {
+	} else if (const auto channel = asChannel()) {
 		appendToIndex(channel->username);
 	}
 	auto toIndex = toIndexList.join(' ');
@@ -344,7 +344,7 @@ void PeerData::fillNames() {
 	const auto namesList = TextUtilities::PrepareSearchWords(toIndex);
 	for (const auto &name : namesList) {
 		_nameWords.insert(name);
-		_nameFirstChars.insert(name[0]);
+		_nameFirstLetters.insert(name[0]);
 	}
 }
 
@@ -361,7 +361,26 @@ const Text &BotCommand::descriptionText() const {
 }
 
 bool UserData::canShareThisContact() const {
-	return canShareThisContactFast() || !App::phoneFromSharedContact(peerToUser(id)).isEmpty();
+	return canShareThisContactFast()
+		|| !Auth().data().findContactPhone(peerToUser(id)).isEmpty();
+}
+
+void UserData::setContactStatus(ContactStatus status) {
+	if (_contactStatus != status) {
+		const auto changed = (_contactStatus == ContactStatus::Contact)
+			!= (status == ContactStatus::Contact);
+		_contactStatus = status;
+		if (changed) {
+			Notify::peerUpdatedDelayed(
+				this,
+				Notify::PeerUpdate::Flag::UserIsContact);
+		}
+	}
+	if (_contactStatus == ContactStatus::Contact
+		&& cReportSpamStatuses().value(id, dbiprsHidden) != dbiprsHidden) {
+		cRefReportSpamStatuses().insert(id, dbiprsHidden);
+		Local::writeReportSpamStatuses();
+	}
 }
 
 // see Local::readPeer as well
@@ -832,13 +851,43 @@ void ChannelData::setAvailableMinId(MsgId availableMinId) {
 		if (auto history = App::historyLoaded(this)) {
 			history->clearUpTill(availableMinId);
 		}
+		if (_pinnedMessageId <= _availableMinId) {
+			_pinnedMessageId = MsgId(0);
+			Notify::peerUpdatedDelayed(
+				this,
+				Notify::PeerUpdate::Flag::ChannelPinnedChanged);
+		}
 	}
 }
 
 void ChannelData::setPinnedMessageId(MsgId messageId) {
+	messageId = (messageId > _availableMinId) ? messageId : MsgId(0);
 	if (_pinnedMessageId != messageId) {
 		_pinnedMessageId = messageId;
-		Notify::peerUpdatedDelayed(this, Notify::PeerUpdate::Flag::ChannelPinnedChanged);
+		Notify::peerUpdatedDelayed(
+			this,
+			Notify::PeerUpdate::Flag::ChannelPinnedChanged);
+	}
+}
+
+void ChannelData::setFeed(not_null<Data::Feed*> feed) {
+	setFeedPointer(feed);
+}
+
+void ChannelData::clearFeed() {
+	setFeedPointer(nullptr);
+}
+
+void ChannelData::setFeedPointer(Data::Feed *feed) {
+	if (_feed != feed) {
+		const auto was = _feed;
+		_feed = feed;
+		if (was) {
+			was->unregisterOne(this);
+		}
+		if (_feed) {
+			_feed->registerOne(this);
+		}
 	}
 }
 
