@@ -1,26 +1,17 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "data/data_search_controller.h"
 
 #include "auth_session.h"
+#include "data/data_session.h"
+#include "data/data_messages.h"
+#include "history/history.h"
+#include "history/history_item.h"
 
 namespace Api {
 namespace {
@@ -35,7 +26,7 @@ MTPmessages_Search PrepareSearchRequest(
 		Storage::SharedMediaType type,
 		const QString &query,
 		MsgId messageId,
-		SparseIdsLoadDirection direction) {
+		Data::LoadDirection direction) {
 	const auto filter = [&] {
 		using Type = Storage::SharedMediaType;
 		switch (type) {
@@ -70,20 +61,21 @@ MTPmessages_Search PrepareSearchRequest(
 	const auto limit = messageId ? kSharedMediaLimit : 0;
 	const auto offsetId = [&] {
 		switch (direction) {
-		case SparseIdsLoadDirection::Before:
-		case SparseIdsLoadDirection::Around: return messageId;
-		case SparseIdsLoadDirection::After: return messageId + 1;
+		case Data::LoadDirection::Before:
+		case Data::LoadDirection::Around: return messageId;
+		case Data::LoadDirection::After: return messageId + 1;
 		}
 		Unexpected("Direction in PrepareSearchRequest");
 	}();
 	const auto addOffset = [&] {
 		switch (direction) {
-		case SparseIdsLoadDirection::Before: return 0;
-		case SparseIdsLoadDirection::Around: return -limit / 2;
-		case SparseIdsLoadDirection::After: return -limit;
+		case Data::LoadDirection::Before: return 0;
+		case Data::LoadDirection::Around: return -limit / 2;
+		case Data::LoadDirection::After: return -limit;
 		}
 		Unexpected("Direction in PrepareSearchRequest");
 	}();
+	const auto hash = int32(0);
 
 	return MTPmessages_Search(
 		MTP_flags(0),
@@ -97,14 +89,15 @@ MTPmessages_Search PrepareSearchRequest(
 		MTP_int(addOffset),
 		MTP_int(limit),
 		MTP_int(maxId),
-		MTP_int(minId));
+		MTP_int(minId),
+		MTP_int(hash));
 }
 
 SearchResult ParseSearchResult(
 		not_null<PeerData*> peer,
 		Storage::SharedMediaType type,
 		MsgId messageId,
-		SparseIdsLoadDirection direction,
+		Data::LoadDirection direction,
 		const MTPmessages_Messages &data) {
 	auto result = SearchResult();
 	result.noSkipRange = MsgRange{ messageId, messageId };
@@ -170,11 +163,11 @@ SearchResult ParseSearchResult(
 	if (messageId && result.messageIds.empty()) {
 		result.noSkipRange = [&]() -> MsgRange {
 			switch (direction) {
-			case SparseIdsLoadDirection::Before: // All old loaded.
+			case Data::LoadDirection::Before: // All old loaded.
 				return { 0, result.noSkipRange.till };
-			case SparseIdsLoadDirection::Around: // All loaded.
+			case Data::LoadDirection::Around: // All loaded.
 				return { 0, ServerMaxMsgId };
-			case SparseIdsLoadDirection::After: // All new loaded.
+			case Data::LoadDirection::After: // All new loaded.
 				return { result.noSkipRange.from, ServerMaxMsgId };
 			}
 			Unexpected("Direction in ParseSearchResult");
