@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "data/data_shared_media.h"
 
@@ -25,12 +12,17 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "apiwrap.h"
 #include "storage/storage_facade.h"
 #include "storage/storage_shared_media.h"
+#include "history/history.h"
+#include "history/history_item.h"
 #include "history/history_media_types.h"
+#include "data/data_media_types.h"
 #include "data/data_sparse_ids.h"
+#include "data/data_session.h"
 #include "info/info_memento.h"
 #include "info/info_controller.h"
 #include "window/window_controller.h"
 #include "mainwindow.h"
+#include "core/crash_reports.h"
 
 namespace {
 
@@ -126,6 +118,14 @@ rpl::producer<SparseIdsSlice> SharedMediaViewer(
 			return builder->removeAll();
 		}) | rpl::start_with_next(pushNextSnapshot, lifetime);
 
+		using InvalidateBottom = Storage::SharedMediaInvalidateBottom;
+		Auth().storage().sharedMediaBottomInvalidated(
+		) | rpl::filter([=](const InvalidateBottom &update) {
+			return (update.peerId == key.peerId);
+		}) | rpl::filter([=] {
+			return builder->invalidateBottom();
+		}) | rpl::start_with_next(pushNextSnapshot, lifetime);
+
 		using Result = Storage::SharedMediaResult;
 		Auth().storage().query(Storage::SharedMediaQuery(
 			key,
@@ -224,7 +224,57 @@ base::optional<int> SharedMediaWithLastSlice::indexOfImpl(Value value) const {
 }
 
 base::optional<int> SharedMediaWithLastSlice::indexOf(Value value) const {
-	auto result = indexOfImpl(value);
+	const auto result = indexOfImpl(value);
+	if (result && (*result < 0 || *result >= size())) {
+		// Should not happen.
+		auto info = QStringList();
+		info.push_back("slice:" + QString::number(_slice.size()));
+		info.push_back(_slice.fullCount()
+			? QString::number(*_slice.fullCount())
+			: QString("-"));
+		info.push_back(_slice.skippedBefore()
+			? QString::number(*_slice.skippedBefore())
+			: QString("-"));
+		info.push_back(_slice.skippedAfter()
+			? QString::number(*_slice.skippedAfter())
+			: QString("-"));
+		info.push_back("ending:" + (_ending
+			? QString::number(_ending->size())
+			: QString("-")));
+		info.push_back((_ending && _ending->fullCount())
+			? QString::number(*_ending->fullCount())
+			: QString("-"));
+		info.push_back((_ending && _ending->skippedBefore())
+			? QString::number(*_ending->skippedBefore())
+			: QString("-"));
+		info.push_back((_ending && _ending->skippedAfter())
+			? QString::number(*_ending->skippedAfter())
+			: QString("-"));
+		if (const auto msgId = base::get_if<FullMsgId>(&value)) {
+			info.push_back("value:" + QString::number(msgId->channel));
+			info.push_back(QString::number(msgId->msg));
+			const auto index = _slice.indexOf(*base::get_if<FullMsgId>(&value));
+			info.push_back("index:" + (index
+				? QString::number(*index)
+				: QString("-")));
+		} else if (const auto photo = base::get_if<not_null<PhotoData*>>(&value)) {
+			info.push_back("value:" + QString::number((*photo)->id));
+		} else {
+			info.push_back("value:bad");
+		}
+		info.push_back("isolated:" + QString(Logs::b(isolatedInSlice())));
+		info.push_back("last:" + (_lastPhotoId
+			? QString::number(*_lastPhotoId)
+			: QString("-")));
+		info.push_back("isolated_last:" + (_isolatedLastPhoto
+			? QString(Logs::b(*_isolatedLastPhoto))
+			: QString("-")));
+		info.push_back("skip:" + (lastPhotoSkip()
+			? QString::number(*lastPhotoSkip())
+			: QString("-")));
+		CrashReports::SetAnnotation("DebugInfo", info.join(','));
+		Unexpected("Result in SharedMediaWithLastSlice::indexOf");
+	}
 	return _reversed
 		? (result | func::negate | func::add(size() - 1))
 		: result;
@@ -243,7 +293,7 @@ SharedMediaWithLastSlice::Value SharedMediaWithLastSlice::operator[](int index)
 	}
 	return (index < _slice.size())
 		? Value(_slice[index])
-		: Value(App::photo(*_lastPhotoId));
+		: Value(Auth().data().photo(*_lastPhotoId));
 }
 
 base::optional<int> SharedMediaWithLastSlice::distance(
@@ -282,10 +332,8 @@ base::optional<bool> SharedMediaWithLastSlice::IsLastIsolated(
 	}
 	return LastFullMsgId(ending ? *ending : slice)
 		| [](FullMsgId msgId) {	return App::histItemById(msgId); }
-		| [](HistoryItem *item) { return item ? item->getMedia() : nullptr; }
-		| [](HistoryMedia *media) {
-			return media ? media->getPhoto() : nullptr;
-		}
+		| [](HistoryItem *item) { return item ? item->media() : nullptr; }
+		| [](Data::Media *media) { return media ? media->photo() : nullptr; }
 		| [](PhotoData *photo) { return photo ? photo->id : 0; }
 		| [&](PhotoId photoId) { return *lastPeerPhotoId != photoId; };
 }
