@@ -1,26 +1,13 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
-#include "dialogs/dialogs_common.h"
+#include "dialogs/dialogs_entry.h"
 #include "dialogs/dialogs_list.h"
 
 class History;
@@ -31,21 +18,26 @@ class IndexedList {
 public:
 	IndexedList(SortMode sortMode);
 
-	RowsByLetter addToEnd(History *history);
-	Row *addByName(History *history);
+	RowsByLetter addToEnd(Key key);
+	Row *addByName(Key key);
 	void adjustByPos(const RowsByLetter &links);
-	void moveToTop(not_null<PeerData*> peer);
+	void moveToTop(Key key);
 
 	// row must belong to this indexed list all().
 	void movePinned(Row *row, int deltaSign);
 
 	// For sortMode != SortMode::Date
-	void peerNameChanged(not_null<PeerData*> peer, const PeerData::NameFirstChars &oldChars);
+	void peerNameChanged(
+		not_null<PeerData*> peer,
+		const base::flat_set<QChar> &oldChars);
 
 	//For sortMode == SortMode::Date
-	void peerNameChanged(Mode list, not_null<PeerData*> peer, const PeerData::NameFirstChars &oldChars);
+	void peerNameChanged(
+		Mode list,
+		not_null<PeerData*> peer,
+		const base::flat_set<QChar> &oldChars);
 
-	void del(not_null<const PeerData*> peer, Row *replacedBy = nullptr);
+	void del(Key key, Row *replacedBy = nullptr);
 	void clear();
 
 	const List &all() const {
@@ -63,8 +55,8 @@ public:
 	// Part of List interface is duplicated here for all() list.
 	int size() const { return all().size(); }
 	bool isEmpty() const { return all().isEmpty(); }
-	bool contains(PeerId peerId) const { return all().contains(peerId); }
-	Row *getRow(PeerId peerId) const { return all().getRow(peerId); }
+	bool contains(Key key) const { return all().contains(key); }
+	Row *getRow(Key key) const { return all().getRow(key); }
 	Row *rowAtY(int32 y, int32 h) const { return all().rowAtY(y, h); }
 
 	using iterator = List::iterator;
@@ -83,8 +75,13 @@ public:
 	iterator find(int y, int h) { return all().find(y, h); }
 
 private:
-	void adjustByName(not_null<PeerData*> peer, const PeerData::NameFirstChars &oldChars);
-	void adjustNames(Mode list, not_null<PeerData*> peer, const PeerData::NameFirstChars &oldChars);
+	void adjustByName(
+		Key key,
+		const base::flat_set<QChar> &oldChars);
+	void adjustNames(
+		Mode list,
+		not_null<History*> history,
+		const base::flat_set<QChar> &oldChars);
 
 	SortMode _sortMode;
 	List _list, _empty;
