@@ -95,7 +95,7 @@ DialogsInner::DialogsInner(QWidget *parent, not_null<Window::Controller*> contro
 		[this](auto item) { itemRemoved(item); },
 		lifetime());
 	Auth().data().itemRepaintRequest(
-	) | rpl::start_with_next([this](auto item) {
+	) | rpl::start_with_next([=](auto item) {
 		const auto history = item->history();
 		if (history->textCachedFor == item) {
 			history->updateChatListEntry();
@@ -114,7 +114,7 @@ DialogsInner::DialogsInner(QWidget *parent, not_null<Window::Controller*> contro
 			UpdateRowSection::Default | UpdateRowSection::Filtered);
 	});
 
-	subscribe(Window::Theme::Background(), [this](const Window::Theme::BackgroundUpdate &data) {
+	subscribe(Window::Theme::Background(), [=](const Window::Theme::BackgroundUpdate &data) {
 		if (data.paletteChanged()) {
 			Dialogs::Layout::clearUnreadBadgesCache();
 		}
