@@ -1,25 +1,16 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
+namespace Data {
+class Feed;
+} // namespace Data
+
 namespace Dialogs {
 
 class Row;
@@ -28,7 +19,11 @@ class FakeRow;
 namespace Layout {
 
 const style::icon *ChatTypeIcon(
-	PeerData *peer,
+	not_null<PeerData*> peer,
+	bool active,
+	bool selected);
+const style::icon *FeedTypeIcon(
+	not_null<Data::Feed*> feed,
 	bool active,
 	bool selected);
 
@@ -36,7 +31,7 @@ class RowPainter {
 public:
 	static void paint(
 		Painter &p,
-		const Row *row,
+		not_null<const Row*> row,
 		int fullWidth,
 		bool active,
 		bool selected,
@@ -44,7 +39,7 @@ public:
 		TimeMs ms);
 	static void paint(
 		Painter &p,
-		const FakeRow *row,
+		not_null<const FakeRow*> row,
 		int fullWidth,
 		bool active,
 		bool selected,
