@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -25,6 +12,8 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 class PeerData;
 namespace Dialogs {
 
+enum class SortMode;
+
 class List {
 public:
 	List(SortMode sortMode);
@@ -37,11 +26,12 @@ public:
 	bool isEmpty() const {
 		return size() == 0;
 	}
-	bool contains(PeerId peerId) const {
-		return _rowByPeer.contains(peerId);
+	bool contains(Key key) const {
+		return _rowByKey.find(key) != _rowByKey.end();
 	}
-	Row *getRow(PeerId peerId) const {
-		return _rowByPeer.value(peerId);
+	Row *getRow(Key key) const {
+		const auto i = _rowByKey.find(key);
+		return (i == _rowByKey.end()) ? nullptr : i->second.get();
 	}
 	Row *rowAtY(int32 y, int32 h) const {
 		auto i = cfind(y, h);
@@ -51,12 +41,12 @@ public:
 		return *i;
 	}
 
-	Row *addToEnd(History *history);
-	Row *adjustByName(const PeerData *peer);
-	Row *addByName(History *history);
-	bool moveToTop(PeerId peerId);
+	Row *addToEnd(Key key);
+	Row *adjustByName(Key key);
+	Row *addByName(Key key);
+	bool moveToTop(Key key);
 	void adjustByPos(Row *row);
-	bool del(PeerId peerId, Row *replacedBy = nullptr);
+	bool del(Key key, Row *replacedBy = nullptr);
 	void remove(Row *row);
 	void clear();
 
@@ -127,10 +117,10 @@ private:
 	SortMode _sortMode;
 	int _count = 0;
 
-	typedef QHash<PeerId, Row*> RowByPeer;
-	RowByPeer _rowByPeer;
+	std::map<Key, not_null<Row*>> _rowByKey;
 
 	mutable Row *_current; // cache
+
 };
 
 } // namespace Dialogs
