@@ -1,27 +1,15 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "dialogs/dialogs_row.h"
 
 #include "styles/style_dialogs.h"
 #include "ui/effects/ripple_animation.h"
+#include "dialogs/dialogs_entry.h"
 #include "mainwidget.h"
 
 namespace Dialogs {
@@ -52,8 +40,12 @@ void RippleRow::paintRipple(Painter &p, int x, int y, int outerWidth, TimeMs ms,
 	}
 }
 
-FakeRow::FakeRow(PeerData *searchInPeer, not_null<HistoryItem*> item)
-: _searchInPeer(searchInPeer)
+uint64 Row::sortKey() const {
+	return _id.entry()->sortKeyInChatList();
+}
+
+FakeRow::FakeRow(Key searchInChat, not_null<HistoryItem*> item)
+: _searchInChat(searchInChat)
 , _item(item)
 , _cache(st::dialogsTextWidthMin) {
 }
