@@ -1,34 +1,24 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
 #include "window/section_widget.h"
 #include "ui/widgets/scroll_area.h"
+#include "dialogs/dialogs_key.h"
 
 class DialogsInner;
 
 namespace Dialogs {
+struct RowDescriptor;
 class Row;
 class FakeRow;
 class IndexedList;
+class Key;
 } // namespace Dialogs
 
 namespace Ui {
@@ -63,13 +53,14 @@ public:
 
 	void updateDragInScroll(bool inScroll);
 
-	void searchInPeer(PeerData *peer);
+	void searchInChat(Dialogs::Key chat);
 
 	void loadDialogs();
 	void loadPinnedDialogs();
-	void createDialog(History *history);
-	void dlgUpdated(Dialogs::Mode list, Dialogs::Row *row);
-	void dlgUpdated(PeerData *peer, MsgId msgId);
+	void createDialog(Dialogs::Key key);
+	void removeDialog(Dialogs::Key key);
+	void repaintDialogRow(Dialogs::Mode list, not_null<Dialogs::Row*> row);
+	void repaintDialogRow(not_null<History*> history, MsgId messageId);
 
 	void dialogsToUp();
 
@@ -84,24 +75,24 @@ public:
 
 	void destroyData();
 
-	void peerBefore(const PeerData *inPeer, MsgId inMsg, PeerData *&outPeer, MsgId &outMsg) const;
-	void peerAfter(const PeerData *inPeer, MsgId inMsg, PeerData *&outPeer, MsgId &outMsg) const;
-	void scrollToPeer(const PeerId &peer, MsgId msgId);
+	Dialogs::RowDescriptor chatListEntryBefore(
+		const Dialogs::RowDescriptor &which) const;
+	Dialogs::RowDescriptor chatListEntryAfter(
+		const Dialogs::RowDescriptor &which) const;
 
-	void removeDialog(History *history);
+	void scrollToPeer(not_null<History*> history, MsgId msgId);
 
 	Dialogs::IndexedList *contactsList();
 	Dialogs::IndexedList *dialogsList();
 	Dialogs::IndexedList *contactsNoDialogsList();
 
-	void searchMessages(const QString &query, PeerData *inPeer = 0);
+	void searchMessages(const QString &query, Dialogs::Key inChat = {});
 	void onSearchMore();
 
 	// Float player interface.
 	bool wheelEventFromFloatPlayer(QEvent *e) override;
 	QRect rectForFloatPlayer() const override;
 
-	void notify_userIsContactChanged(UserData *user, bool fromThisApp);
 	void notify_historyMuteUpdated(History *history);
 
 signals:
@@ -115,7 +106,7 @@ public slots:
 	void activate();
 	void onFilterUpdate(bool force = false);
 	bool onCancelSearch();
-	void onCancelSearchInPeer();
+	void onCancelSearchInChat();
 
 	void onFilterCursorMoved(int from = -1, int to = -1);
 	void onCompleteHashtag(QString tag);
@@ -144,13 +135,29 @@ protected:
 
 private:
 	void animationCallback();
-	void dialogsReceived(const MTPmessages_Dialogs &dialogs, mtpRequestId requestId);
-	void pinnedDialogsReceived(const MTPmessages_PeerDialogs &dialogs, mtpRequestId requestId);
-	void contactsReceived(const MTPcontacts_Contacts &result);
-	void searchReceived(DialogsSearchRequestType type, const MTPmessages_Messages &result, mtpRequestId requestId);
-	void peerSearchReceived(const MTPcontacts_Found &result, mtpRequestId requestId);
+	void dialogsReceived(
+		const MTPmessages_Dialogs &result,
+		mtpRequestId requestId);
+	void pinnedDialogsReceived(
+		const MTPmessages_PeerDialogs &result,
+		mtpRequestId requestId);
+	void searchReceived(
+		DialogsSearchRequestType type,
+		const MTPmessages_Messages &result,
+		mtpRequestId requestId);
+	void peerSearchReceived(
+		const MTPcontacts_Found &result,
+		mtpRequestId requestId);
+	void updateDialogsOffset(
+		const QVector<MTPDialog> &dialogs,
+		const QVector<MTPMessage> &messages);
+	void applyReceivedDialogs(
+		const QVector<MTPDialog> &dialogs,
+		const QVector<MTPMessage> &messages);
 
-	void setSearchInPeer(PeerData *peer, UserData *from = nullptr);
+	bool searchForPeersRequired(const QString &query) const;
+	void setSearchInChat(Dialogs::Key chat, UserData *from = nullptr);
+	void showJumpToDate();
 	void showSearchFrom();
 	void showMainMenu();
 	void clearSearchCache();
@@ -160,9 +167,7 @@ private:
 	void updateControlsGeometry();
 	void updateForwardBar();
 
-	void unreadCountsReceived(const QVector<MTPDialog> &dialogs);
 	bool dialogsFailed(const RPCError &error, mtpRequestId req);
-	bool contactsFailed(const RPCError &error);
 	bool searchFailed(DialogsSearchRequestType type, const RPCError &error, mtpRequestId req);
 	bool peopleFailed(const RPCError &error, mtpRequestId req);
 
@@ -176,7 +181,6 @@ private:
 	PeerData *_dialogsOffsetPeer = nullptr;
 	mtpRequestId _dialogsRequestId = 0;
 	mtpRequestId _pinnedDialogsRequestId = 0;
-	mtpRequestId _contactsRequestId = 0;
 	bool _pinnedDialogsReceived = false;
 
 	object_ptr<Ui::IconButton> _forwardCancel = { nullptr };
@@ -195,8 +199,8 @@ private:
 	Window::SlideDirection _showDirection;
 	QPixmap _cacheUnder, _cacheOver;
 
-	PeerData *_searchInPeer = nullptr;
-	PeerData *_searchInMigrated = nullptr;
+	Dialogs::Key _searchInChat;
+	History *_searchInMigrated = nullptr;
 	UserData *_searchFromUser = nullptr;
 	QString _lastFilterText;
 
