@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "facades.h"
 
@@ -35,12 +22,11 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "window/layer_widget.h"
 #include "lang/lang_keys.h"
 #include "base/observer.h"
-#include "base/task_queue.h"
+#include "history/history.h"
+#include "history/history_item.h"
 #include "history/history_media.h"
 #include "styles/style_history.h"
-
-Q_DECLARE_METATYPE(ClickHandlerPtr);
-Q_DECLARE_METATYPE(Qt::MouseButton);
+#include "data/data_session.h"
 
 namespace App {
 namespace internal {
@@ -163,13 +149,17 @@ void activateBotCommand(
 }
 
 void searchByHashtag(const QString &tag, PeerData *inPeer) {
-	if (MainWidget *m = main()) {
+	if (const auto m = main()) {
 		Ui::hideSettingsAndLayer();
 		Messenger::Instance().hideMediaView();
 		if (inPeer && (!inPeer->isChannel() || inPeer->isMegagroup())) {
 			inPeer = nullptr;
 		}
-		m->searchMessages(tag + ' ', inPeer);
+		m->searchMessages(
+			tag + ' ',
+			(inPeer
+				? App::history(inPeer).get()
+				: Dialogs::Key()));
 	}
 }
 
@@ -181,12 +171,6 @@ void joinGroupByHash(const QString &hash) {
 	if (MainWidget *m = main()) m->joinGroupByHash(hash);
 }
 
-void removeDialog(History *history) {
-	if (MainWidget *m = main()) {
-		m->removeDialog(history);
-	}
-}
-
 void showSettings() {
 	if (auto w = wnd()) {
 		w->showSettings();
@@ -194,11 +178,9 @@ void showSettings() {
 }
 
 void activateClickHandler(ClickHandlerPtr handler, Qt::MouseButton button) {
-	if (auto w = wnd()) {
-		qRegisterMetaType<ClickHandlerPtr>();
-		qRegisterMetaType<Qt::MouseButton>();
-		QMetaObject::invokeMethod(w, "app_activateClickHandler", Qt::QueuedConnection, Q_ARG(ClickHandlerPtr, handler), Q_ARG(Qt::MouseButton, button));
-	}
+	crl::on_main(wnd(), [handler, button] {
+		handler->onClick(button);
+	});
 }
 
 void logOutDelayed() {
@@ -261,18 +243,6 @@ bool isLayerShown() {
 	return false;
 }
 
-void autoplayMediaInlineAsync(const FullMsgId &msgId) {
-	if (auto main = App::main()) {
-		InvokeQueued(main, [msgId] {
-			if (auto item = App::histItemById(msgId)) {
-				if (auto media = item->getMedia()) {
-					media->playInline(true);
-				}
-			}
-		});
-	}
-}
-
 void showPeerProfile(const PeerId &peer) {
 	if (auto window = App::wnd()) {
 		if (auto controller = window->controller()) {
@@ -281,18 +251,28 @@ void showPeerProfile(const PeerId &peer) {
 	}
 }
 
+void showPeerProfile(not_null<const History*> history) {
+	showPeerProfile(history->peer->id);
+}
+
 void showPeerHistory(
 		const PeerId &peer,
 		MsgId msgId) {
 	auto ms = getms();
-	LOG(("Show Peer Start"));
 	if (auto m = App::main()) {
 		m->ui_showPeerHistory(
 			peer,
 			Window::SectionShow::Way::ClearStack,
 			msgId);
 	}
-	LOG(("Show Peer End: %1").arg(getms() - ms));
+}
+
+void showPeerHistoryAtItem(not_null<const HistoryItem*> item) {
+	showPeerHistory(item->history()->peer->id, item->id);
+}
+
+void showPeerHistory(not_null<const History*> history, MsgId msgId) {
+	showPeerHistory(history->peer->id, msgId);
 }
 
 PeerData *getPeerForMouseAction() {
@@ -316,10 +296,6 @@ void userIsBotChanged(UserData *user) {
 	if (MainWidget *m = App::main()) m->notify_userIsBotChanged(user);
 }
 
-void userIsContactChanged(UserData *user, bool fromThisApp) {
-	if (MainWidget *m = App::main()) m->notify_userIsContactChanged(user, fromThisApp);
-}
-
 void botCommandsChanged(UserData *user) {
 	if (MainWidget *m = App::main()) {
 		m->notify_botCommandsChanged(user);
@@ -358,38 +334,6 @@ void historyMuteUpdated(History *history) {
 	if (MainWidget *m = App::main()) m->notify_historyMuteUpdated(history);
 }
 
-void handlePendingHistoryUpdate() {
-	if (!AuthSession::Exists()) {
-		return;
-	}
-	Auth().data().pendingHistoryResize().notify(true);
-
-	for (const auto item : base::take(Global::RefPendingRepaintItems())) {
-		Auth().data().requestItemRepaint(item);
-
-		// Start the video if it is waiting for that.
-		if (item->pendingInitDimensions()) {
-			if (const auto media = item->getMedia()) {
-				if (const auto reader = media->getClipReader()) {
-					const auto startRequired = [&] {
-						if (reader->started()) {
-							return false;
-						}
-						using Mode = Media::Clip::Reader::Mode;
-						return (reader->mode() == Mode::Video);
-					};
-					if (startRequired()) {
-						const auto width = std::max(
-							item->width(),
-							st::historyMinimalWidth);
-						item->resizeGetHeight(width);
-					}
-				}
-			}
-		}
-	}
-}
-
 void unreadCounterUpdated() {
 	Global::RefHandleUnreadCounterUpdate().call();
 }
@@ -508,20 +452,7 @@ void WorkingDirReady() {
 	}
 }
 
-object_ptr<SingleQueuedInvokation> MainThreadTaskHandler = { nullptr };
-
-void MainThreadTaskAdded() {
-	if (!started()) {
-		return;
-	}
-
-	MainThreadTaskHandler->call();
-}
-
 void start() {
-	MainThreadTaskHandler.create([] {
-		base::TaskQueue::ProcessMainTasks();
-	});
 	SandboxData = std::make_unique<internal::Data>();
 }
 
@@ -531,7 +462,6 @@ bool started() {
 
 void finish() {
 	SandboxData.reset();
-	MainThreadTaskHandler.destroy();
 }
 
 uint64 UserTag() {
@@ -547,7 +477,6 @@ namespace Global {
 namespace internal {
 
 struct Data {
-	SingleQueuedInvokation HandleHistoryUpdate = { [] { Messenger::Instance().call_handleHistoryUpdate(); } };
 	SingleQueuedInvokation HandleUnreadCounterUpdate = { [] { Messenger::Instance().call_handleUnreadCounterUpdate(); } };
 	SingleQueuedInvokation HandleDelayedPeerUpdates = { [] { Messenger::Instance().call_handleDelayedPeerUpdates(); } };
 	SingleQueuedInvokation HandleObservables = { [] { Messenger::Instance().call_handleObservables(); } };
@@ -582,11 +511,13 @@ struct Data {
 	int32 OnlineCloudTimeout = 300000;
 	int32 NotifyCloudDelay = 30000;
 	int32 NotifyDefaultDelay = 1500;
-	int32 ChatBigSize = 10;
 	int32 PushChatPeriod = 60000;
 	int32 PushChatLimit = 2;
 	int32 SavedGifsLimit = 200;
 	int32 EditTimeLimit = 172800;
+	int32 RevokeTimeLimit = 172800;
+	int32 RevokePrivateTimeLimit = 172800;
+	bool RevokePrivateInbox = false;
 	int32 StickersRecentLimit = 30;
 	int32 StickersFavedLimit = 5;
 	int32 PinnedDialogsCountMax = 5;
@@ -601,8 +532,6 @@ struct Data {
 
 	HiddenPinnedMessagesMap HiddenPinnedMessages;
 
-	PendingItemsMap PendingRepaintItems;
-
 	Stickers::Sets StickerSets;
 	Stickers::Order StickerSetsOrder;
 	TimeMs LastStickersUpdate = 0;
@@ -623,6 +552,7 @@ struct Data {
 	QByteArray DownloadPathBookmark;
 	base::Observable<void> DownloadPathChanged;
 
+	bool SuggestStickersByEmoji = true;
 	bool SoundNotify = true;
 	bool DesktopNotify = true;
 	bool RestoreSoundNotifyFromTray = false;
@@ -670,7 +600,6 @@ void finish() {
 	GlobalData = nullptr;
 }
 
-DefineRefVar(Global, SingleQueuedInvokation, HandleHistoryUpdate);
 DefineRefVar(Global, SingleQueuedInvokation, HandleUnreadCounterUpdate);
 DefineRefVar(Global, SingleQueuedInvokation, HandleDelayedPeerUpdates);
 DefineRefVar(Global, SingleQueuedInvokation, HandleObservables);
@@ -705,11 +634,13 @@ DefineVar(Global, int32, OnlineFocusTimeout);
 DefineVar(Global, int32, OnlineCloudTimeout);
 DefineVar(Global, int32, NotifyCloudDelay);
 DefineVar(Global, int32, NotifyDefaultDelay);
-DefineVar(Global, int32, ChatBigSize);
 DefineVar(Global, int32, PushChatPeriod);
 DefineVar(Global, int32, PushChatLimit);
 DefineVar(Global, int32, SavedGifsLimit);
 DefineVar(Global, int32, EditTimeLimit);
+DefineVar(Global, int32, RevokeTimeLimit);
+DefineVar(Global, int32, RevokePrivateTimeLimit);
+DefineVar(Global, bool, RevokePrivateInbox);
 DefineVar(Global, int32, StickersRecentLimit);
 DefineVar(Global, int32, StickersFavedLimit);
 DefineVar(Global, int32, PinnedDialogsCountMax);
@@ -724,8 +655,6 @@ DefineRefVar(Global, base::Observable<void>, PhoneCallsEnabledChanged);
 
 DefineVar(Global, HiddenPinnedMessagesMap, HiddenPinnedMessages);
 
-DefineRefVar(Global, PendingItemsMap, PendingRepaintItems);
-
 DefineVar(Global, Stickers::Sets, StickerSets);
 DefineVar(Global, Stickers::Order, StickerSetsOrder);
 DefineVar(Global, TimeMs, LastStickersUpdate);
@@ -746,6 +675,7 @@ DefineVar(Global, QString, DownloadPath);
 DefineVar(Global, QByteArray, DownloadPathBookmark);
 DefineRefVar(Global, base::Observable<void>, DownloadPathChanged);
 
+DefineVar(Global, bool, SuggestStickersByEmoji);
 DefineVar(Global, bool, SoundNotify);
 DefineVar(Global, bool, DesktopNotify);
 DefineVar(Global, bool, RestoreSoundNotifyFromTray);
