@@ -1,26 +1,16 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
+#include "history/view/history_view_object.h"
+
 struct HistoryMessageEdited;
+struct TextSelection;
 
 namespace base {
 template <typename Enum>
@@ -28,10 +18,18 @@ class enum_mask;
 } // namespace base
 
 namespace Storage {
-enum class SharedMediaType : char;
+enum class SharedMediaType : signed char;
 using SharedMediaTypesMask = base::enum_mask<SharedMediaType>;
 } // namespace Storage
 
+namespace HistoryView {
+enum class PointState : char;
+enum class CursorState : char;
+enum class InfoDisplayType : char;
+struct TextState;
+struct StateRequest;
+} // namespace HistoryView
+
 enum class MediaInBubbleState {
 	None,
 	Top,
@@ -39,52 +37,58 @@ enum class MediaInBubbleState {
 	Bottom,
 };
 
-class HistoryMedia : public HistoryElement {
+enum HistoryMediaType : char {
+	MediaTypePhoto,
+	MediaTypeVideo,
+	MediaTypeContact,
+	MediaTypeCall,
+	MediaTypeFile,
+	MediaTypeGif,
+	MediaTypeSticker,
+	MediaTypeLocation,
+	MediaTypeWebPage,
+	MediaTypeMusicFile,
+	MediaTypeVoiceFile,
+	MediaTypeGame,
+	MediaTypeInvoice,
+	MediaTypeGrouped,
+
+	MediaTypeCount
+};
+
+class HistoryMedia : public HistoryView::Object {
 public:
-	HistoryMedia(not_null<HistoryItem*> parent) : _parent(parent) {
+	using Element = HistoryView::Element;
+	using PointState = HistoryView::PointState;
+	using TextState = HistoryView::TextState;
+	using StateRequest = HistoryView::StateRequest;
+
+	HistoryMedia(not_null<Element*> parent) : _parent(parent) {
 	}
 
 	virtual HistoryMediaType type() const = 0;
 
-	virtual QString notificationText() const {
-		return QString();
+	virtual TextWithEntities selectedText(TextSelection selection) const {
+		return TextWithEntities();
 	}
 
-	// Returns text with link-start and link-end commands for service-color highlighting.
-	// Example: "[link1-start]You:[link1-end] [link1-start]Photo,[link1-end] caption text"
-	virtual QString inDialogsText() const {
-		auto result = notificationText();
-		return result.isEmpty() ? QString() : textcmdLink(1, TextUtilities::Clean(result));
-	}
-	virtual TextWithEntities selectedText(TextSelection selection) const = 0;
-
-	bool hasPoint(QPoint point) const {
-		return QRect(0, 0, _width, _height).contains(point);
-	}
-
-	virtual bool isDisplayed() const {
-		return !_parent->isHiddenByGroup();
-	}
+	virtual bool isDisplayed() const;
 	virtual void updateNeedBubbleState() {
 	}
-	virtual bool isAboveMessage() const {
-		return false;
-	}
 	virtual bool hasTextForCopy() const {
 		return false;
 	}
+	virtual bool hideMessageText() const {
+		return true;
+	}
 	virtual bool allowsFastShare() const {
 		return false;
 	}
-	virtual void initDimensions() = 0;
 	virtual void refreshParentId(not_null<HistoryItem*> realParent) {
 	}
-	virtual int resizeGetHeight(int width) {
-		_width = qMin(width, _maxw);
-		return _height;
-	}
 	virtual void draw(Painter &p, const QRect &r, TextSelection selection, TimeMs ms) const = 0;
-	virtual HistoryTextState getState(QPoint point, HistoryStateRequest request) const = 0;
+	virtual PointState pointState(QPoint point) const;
+	virtual TextState textState(QPoint point, StateRequest request) const = 0;
 	virtual void updatePressed(QPoint point) {
 	}
 
@@ -105,25 +109,13 @@ public:
 			TextSelectType type) const {
 		return selection;
 	}
-	[[nodiscard]] virtual bool consumeMessageText(
-			const TextWithEntities &textWithEntities) {
-		return false;
-	}
 	[[nodiscard]] virtual uint16 fullSelectionLength() const {
 		return 0;
 	}
 	[[nodiscard]] TextSelection skipSelection(
-			TextSelection selection) const {
-		return internal::unshiftSelection(
-			selection,
-			fullSelectionLength());
-	}
+		TextSelection selection) const;
 	[[nodiscard]] TextSelection unskipSelection(
-			TextSelection selection) const {
-		return internal::shiftSelection(
-			selection,
-			fullSelectionLength());
-	}
+		TextSelection selection) const;
 
 	// if we press and drag this link should we drag the item
 	virtual bool dragItemByHandler(const ClickHandlerPtr &p) const = 0;
@@ -136,9 +128,6 @@ public:
 	virtual bool uploading() const {
 		return false;
 	}
-	virtual std::unique_ptr<HistoryMedia> clone(
-		not_null<HistoryItem*> newParent,
-		not_null<HistoryItem*> realParent) const = 0;
 
 	virtual PhotoData *getPhoto() const {
 		return nullptr;
@@ -146,30 +135,16 @@ public:
 	virtual DocumentData *getDocument() const {
 		return nullptr;
 	}
-	virtual Media::Clip::Reader *getClipReader() {
-		return nullptr;
+
+	void playAnimation() {
+		playAnimation(false);
+	}
+	void autoplayAnimation() {
+		playAnimation(true);
+	}
+	virtual void stopAnimation() {
 	}
 
-	bool playInline(/*bool autoplay = false*/) {
-		return playInline(false);
-	}
-	virtual bool playInline(bool autoplay) {
-		return false;
-	}
-	virtual void stopInline() {
-	}
-	virtual bool isRoundVideoPlaying() const {
-		return false;
-	}
-
-	virtual void attachToParent() {
-	}
-	virtual void detachFromParent() {
-	}
-
-	virtual bool canBeGrouped() const {
-		return false;
-	}
 	virtual QSize sizeForGrouping() const {
 		Unexpected("Grouping method call.");
 	}
@@ -184,39 +159,15 @@ public:
 			not_null<QPixmap*> cache) const {
 		Unexpected("Grouping method call.");
 	}
-	virtual HistoryTextState getStateGrouped(
-			const QRect &geometry,
-			QPoint point,
-			HistoryStateRequest request) const {
-		Unexpected("Grouping method call.");
-	}
-	virtual std::unique_ptr<HistoryMedia> takeLastFromGroup() {
-		return nullptr;
-	}
-	virtual bool applyGroup(
-			const std::vector<not_null<HistoryItem*>> &others) {
-		return others.empty();
-	}
-
-	virtual void updateSentMedia(const MTPMessageMedia &media) {
-	}
-
-	// After sending an inline result we may want to completely recreate
-	// the media (all media that was generated on client side, for example)
-	virtual bool needReSetInlineResultMedia(const MTPMessageMedia &media) {
-		return true;
-	}
+	virtual TextState getStateGrouped(
+		const QRect &geometry,
+		QPoint point,
+		StateRequest request) const;
 
 	virtual bool animating() const {
 		return false;
 	}
 
-	virtual bool hasReplyPreview() const {
-		return false;
-	}
-	virtual ImagePtr replyPreview() {
-		return ImagePtr();
-	}
 	virtual TextWithEntities getCaption() const {
 		return TextWithEntities();
 	}
@@ -243,10 +194,6 @@ public:
 		return QString();
 	}
 
-	int currentWidth() const {
-		return _width;
-	}
-
 	void setInBubbleState(MediaInBubbleState state) {
 		_inBubbleState = state;
 	}
@@ -263,23 +210,34 @@ public:
 		return false;
 	}
 
-	virtual bool canEditCaption() const {
-		return false;
-	}
-
-	// Sometimes click on media in message is overloaded by the messsage:
+	// Sometimes click on media in message is overloaded by the message:
 	// (for example it can open a link or a game instead of opening media)
 	// But the overloading click handler should be used only when media
-	// is already loaded (not a photo or gif waiting for load with auto
+	// is already loaded (not a photo or GIF waiting for load with auto
 	// load being disabled - in such case media should handle the click).
 	virtual bool isReadyForOpen() const {
 		return true;
 	}
 
+	// Should be called only by Data::Session.
+	virtual void updateSharedContactUserId(UserId userId) {
+	}
+	virtual void parentTextUpdated() {
+	}
+
+	virtual ~HistoryMedia() = default;
+
 protected:
-	not_null<HistoryItem*> _parent;
-	int _width = 0;
+	using CursorState = HistoryView::CursorState;
+	using InfoDisplayType = HistoryView::InfoDisplayType;
+
+	QSize countCurrentSize(int newWidth) override;
+	Text createCaption(not_null<HistoryItem*> item) const;
+
+	virtual void playAnimation(bool autoplay) {
+	}
+
+	not_null<Element*> _parent;
 	MediaInBubbleState _inBubbleState = MediaInBubbleState::None;
 
-
 };
