@@ -1,28 +1,17 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "info/profile/info_profile_actions.h"
 
 #include <rpl/flatten_latest.h>
 #include <rpl/combine.h>
 #include "data/data_peer_values.h"
+#include "data/data_session.h"
+#include "data/data_feed.h"
 #include "ui/wrap/vertical_layout.h"
 #include "ui/wrap/padding_wrap.h"
 #include "ui/wrap/slide_wrap.h"
@@ -177,6 +166,37 @@ private:
 
 };
 
+class FeedDetailsFiller {
+public:
+	FeedDetailsFiller(
+		not_null<Controller*> controller,
+		not_null<Ui::RpWidget*> parent,
+		not_null<Data::Feed*> feed);
+
+	object_ptr<Ui::RpWidget> fill();
+
+private:
+	object_ptr<Ui::RpWidget> setupDefaultToggle();
+
+	template <
+		typename Widget,
+		typename = std::enable_if_t<
+		std::is_base_of_v<Ui::RpWidget, Widget>>>
+	Widget *add(
+			object_ptr<Widget> &&child,
+			const style::margins &margin = style::margins()) {
+		return _wrap->add(
+			std::move(child),
+			margin);
+	}
+
+	not_null<Controller*> _controller;
+	not_null<Ui::RpWidget*> _parent;
+	not_null<Data::Feed*> _feed;
+	object_ptr<Ui::VerticalLayout> _wrap;
+
+};
+
 DetailsFiller::DetailsFiller(
 	not_null<Controller*> controller,
 	not_null<Ui::RpWidget*> parent,
@@ -329,9 +349,13 @@ Ui::MultiSlideTracker DetailsFiller::fillUserButtons(
 	auto window = _controller->parentController();
 
 	auto addSendMessageButton = [&] {
+		auto activePeerValue = window->activeChatValue(
+		) | rpl::map([](Dialogs::Key key) {
+			return key.peer();
+		});
 		auto sendMessageVisible = rpl::combine(
 			_controller->wrapValue(),
-			window->historyPeer.value(),
+			std::move(activePeerValue),
 			(_1 != Wrap::Side) || (_2 != user));
 		auto sendMessage = [window, user] {
 			window->showPeerHistory(
@@ -379,9 +403,13 @@ Ui::MultiSlideTracker DetailsFiller::fillChannelButtons(
 
 	Ui::MultiSlideTracker tracker;
 	auto window = _controller->parentController();
+	auto activePeerValue = window->activeChatValue(
+	) | rpl::map([](Dialogs::Key key) {
+		return key.peer();
+	});
 	auto viewChannelVisible = rpl::combine(
 		_controller->wrapValue(),
-		window->historyPeer.value(),
+		std::move(activePeerValue),
 		(_1 != Wrap::Side) || (_2 != channel));
 	auto viewChannel = [=] {
 		window->showPeerHistory(
@@ -562,19 +590,11 @@ void ActionsFiller::addBlockAction(not_null<UserData*> user) {
 
 void ActionsFiller::addLeaveChannelAction(
 		not_null<ChannelData*> channel) {
-	auto callback = [=] {
-		auto text = lang(channel->isMegagroup()
-			? lng_sure_leave_group
-			: lng_sure_leave_channel);
-		Ui::show(Box<ConfirmBox>(text, lang(lng_box_leave), [=] {
-			Auth().api().leaveChannel(channel);
-		}), LayerOption::KeepOther);
-	};
 	AddActionButton(
 		_wrap,
 		Lang::Viewer(lng_profile_leave_channel),
 		AmInChannelValue(channel),
-		std::move(callback));
+		Window::DeleteAndLeaveHandler(channel));
 }
 
 void ActionsFiller::addJoinChannelAction(
@@ -663,6 +683,47 @@ object_ptr<Ui::RpWidget> ActionsFiller::fill() {
 	return { nullptr };
 }
 
+FeedDetailsFiller::FeedDetailsFiller(
+	not_null<Controller*> controller,
+	not_null<Ui::RpWidget*> parent,
+	not_null<Data::Feed*> feed)
+: _controller(controller)
+, _parent(parent)
+, _feed(feed)
+, _wrap(_parent) {
+}
+
+object_ptr<Ui::RpWidget> FeedDetailsFiller::fill() {
+	add(object_ptr<BoxContentDivider>(_wrap));
+	add(CreateSkipWidget(_wrap));
+	add(setupDefaultToggle());
+	add(CreateSkipWidget(_wrap));
+	return std::move(_wrap);
+}
+
+object_ptr<Ui::RpWidget> FeedDetailsFiller::setupDefaultToggle() {
+	using namespace rpl::mappers;
+	const auto feedId = _feed->id();
+	auto result = object_ptr<Button>(
+		_wrap,
+		Lang::Viewer(lng_info_feed_is_default),
+		st::infoNotificationsButton);
+	result->toggleOn(
+		Auth().data().defaultFeedIdValue(
+		) | rpl::map(_1 == feedId)
+	)->addClickHandler([=] {
+		const auto makeDefault = (Auth().data().defaultFeedId() != feedId);
+		const auto defaultFeedId = makeDefault ? feedId : 0;
+		Auth().data().setDefaultFeedId(defaultFeedId);
+//		Auth().api().saveDefaultFeedId(feedId, makeDefault); // #feed
+	});
+	object_ptr<FloatingIcon>(
+		result,
+		st::infoIconNotifications,
+		st::infoNotificationsIconPosition);
+	return std::move(result);
+}
+
 } // namespace
 
 object_ptr<Ui::RpWidget> SetupDetails(
@@ -759,5 +820,13 @@ object_ptr<Ui::RpWidget> SetupChannelMembers(
 	return std::move(result);
 }
 
+object_ptr<Ui::RpWidget> SetupFeedDetails(
+		not_null<Controller*> controller,
+		not_null<Ui::RpWidget*> parent,
+		not_null<Data::Feed*> feed) {
+	FeedDetailsFiller filler(controller, parent, feed);
+	return filler.fill();
+}
+
 } // namespace Profile
 } // namespace Info
