@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -128,13 +115,19 @@ private:
 
 };
 
-using DocumentItems = QMap<DocumentData*, OrderedSet<ItemBase*>>;
+using DocumentItems = std::map<
+	not_null<const DocumentData*>,
+	base::flat_set<not_null<ItemBase*>>>;
 const DocumentItems *documentItems();
 
 namespace internal {
 
-void regDocumentItem(DocumentData *document, ItemBase *item);
-void unregDocumentItem(DocumentData *document, ItemBase *item);
+void regDocumentItem(
+	not_null<const DocumentData*> document,
+	not_null<ItemBase*> item);
+void unregDocumentItem(
+	not_null<const DocumentData*> document,
+	not_null<ItemBase*> item);
 
 } // namespace internal
 } // namespace Layout
