@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -76,9 +63,12 @@ public:
 	~Result();
 
 private:
-	void createPhoto();
-	void createDocument();
 	void createGame();
+	QSize thumbBox() const;
+	MTPWebDocument adjustAttributes(const MTPWebDocument &document);
+	MTPVector<MTPDocumentAttribute> adjustAttributes(
+		const MTPVector<MTPDocumentAttribute> &document,
+		const MTPstring &mimeType);
 
 	enum class Type {
 		Unknown,
@@ -105,12 +95,8 @@ private:
 	uint64 _queryId = 0;
 	QString _id;
 	Type _type = Type::Unknown;
-	QString _title, _description, _url, _thumb_url;
-	QString _content_type, _content_url;
-	int _width = 0;
-	int _height = 0;
-	int _duration = 0;
-
+	QString _title, _description, _url;
+	QString _content_url;
 	DocumentData *_document = nullptr;
 	PhotoData *_photo = nullptr;
 	GameData *_game = nullptr;
