@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -100,9 +87,12 @@ public:
 // Plain text message.
 class SendText : public SendDataCommon {
 public:
-	SendText(const QString &message, const EntitiesInText &entities, bool/* noWebPage*/)
-		: _message(message)
-		, _entities(entities) {
+	SendText(
+		const QString &message,
+		const EntitiesInText &entities,
+		bool/* noWebPage*/)
+	: _message(message)
+	, _entities(entities) {
 	}
 
 	bool isValid() const override {
@@ -201,9 +191,13 @@ private:
 // Message with photo.
 class SendPhoto : public SendData {
 public:
-	SendPhoto(PhotoData *photo, const QString &caption)
-		: _photo(photo)
-		, _caption(caption) {
+	SendPhoto(
+		PhotoData *photo,
+		const QString &message,
+		const EntitiesInText &entities)
+	: _photo(photo)
+	, _message(message)
+	, _entities(entities) {
 	}
 
 	bool isValid() const override {
@@ -228,16 +222,21 @@ public:
 
 private:
 	PhotoData *_photo;
-	QString _caption;
+	QString _message;
+	EntitiesInText _entities;
 
 };
 
 // Message with file.
 class SendFile : public SendData {
 public:
-	SendFile(DocumentData *document, const QString &caption)
-		: _document(document)
-		, _caption(caption) {
+	SendFile(
+		DocumentData *document,
+		const QString &message,
+		const EntitiesInText &entities)
+	: _document(document)
+	, _message(message)
+	, _entities(entities) {
 	}
 
 	bool isValid() const override {
@@ -262,7 +261,8 @@ public:
 
 private:
 	DocumentData *_document;
-	QString _caption;
+	QString _message;
+	EntitiesInText _entities;
 
 };
 
@@ -270,7 +270,7 @@ private:
 class SendGame : public SendData {
 public:
 	SendGame(GameData *game)
-		: _game(game) {
+	: _game(game) {
 	}
 
 	bool isValid() const override {
