@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -38,7 +25,10 @@ struct PeerUpdate;
 } // namespace Notify
 
 namespace Dialogs {
+struct RowDescriptor;
 class Row;
+class Key;
+class IndexedList;
 } // namespace Dialogs
 
 namespace Media {
@@ -118,11 +108,10 @@ public:
 
 	void activate();
 
-	void createDialog(History *history);
-	void removeDialog(History *history);
-	void dlgUpdated();
-	void dlgUpdated(Dialogs::Mode list, Dialogs::Row *row);
-	void dlgUpdated(PeerData *peer, MsgId msgId);
+	void createDialog(Dialogs::Key key);
+	void removeDialog(Dialogs::Key key);
+	void repaintDialogRow(Dialogs::Mode list, not_null<Dialogs::Row*> row);
+	void repaintDialogRow(not_null<History*> history, MsgId messageId);
 
 	void windowShown();
 
@@ -136,15 +125,17 @@ public:
 		const MTPUpdates &updates);
 	void historyToDown(History *hist);
 	void dialogsToUp();
-	void newUnreadMsg(History *history, HistoryItem *item);
+	void newUnreadMsg(
+		not_null<History*> history,
+		not_null<HistoryItem*> item);
 	void markActiveHistoryAsRead();
 
-	void peerBefore(const PeerData *inPeer, MsgId inMsg, PeerData *&outPeer, MsgId &outMsg);
-	void peerAfter(const PeerData *inPeer, MsgId inMsg, PeerData *&outPeer, MsgId &outMsg);
-	PeerData *peer();
+	Dialogs::RowDescriptor chatListEntryBefore(
+		const Dialogs::RowDescriptor &which) const;
+	Dialogs::RowDescriptor chatListEntryAfter(
+		const Dialogs::RowDescriptor &which) const;
 
-	PeerData *activePeer();
-	MsgId activeMsgId();
+	PeerData *peer();
 
 	int backgroundFromY() const;
 	void showSection(
@@ -179,18 +170,18 @@ public:
 
 	void showForwardLayer(MessageIdsList &&items);
 	void showSendPathsLayer();
-	void deleteLayer(int selectedCount = 0); // 0 - context item
-	void cancelUploadLayer();
+	void deleteLayer(FullMsgId itemId);
+	void cancelUploadLayer(not_null<HistoryItem*> item);
 	void shareUrlLayer(const QString &url, const QString &text);
 	void inlineSwitchLayer(const QString &botAndQuery);
 	void hiderLayer(object_ptr<HistoryHider> h);
 	void noHider(HistoryHider *destroyed);
-	bool setForwardDraft(PeerId peer, ForwardWhatMessages what);
 	bool setForwardDraft(PeerId peer, MessageIdsList &&items);
 	bool shareUrl(
 		not_null<PeerData*> peer,
 		const QString &url,
 		const QString &text);
+	void replyToItem(not_null<HistoryItem*> item);
 	bool onInlineSwitchChosen(const PeerId &peer, const QString &botAndQuery);
 	bool onSendPaths(const PeerId &peer);
 	void onFilesOrForwardDrop(const PeerId &peer, const QMimeData *data);
@@ -208,7 +199,9 @@ public:
 		const QVector<MTPint> &ids,
 		bool forEveryone);
 	void deletedContact(UserData *user, const MTPcontacts_Link &result);
-	void deleteConversation(PeerData *peer, bool deleteHistory = true);
+	void deleteConversation(
+		not_null<PeerData*> peer,
+		bool deleteHistory = true);
 	void deleteAndExit(ChatData *chat);
 	void deleteAllFromUser(ChannelData *channel, UserData *from);
 
@@ -224,15 +217,8 @@ public:
 		not_null<ChannelData*> channel,
 		const RPCError &e); // for multi invite in channels
 
-	void checkPeerHistory(PeerData *peer);
-	void checkedHistory(PeerData *peer, const MTPmessages_Messages &result);
-
 	bool sendMessageFail(const RPCError &error);
 
-	void forwardSelectedItems();
-	void confirmDeleteSelectedItems();
-	void clearSelectedItems();
-
 	Dialogs::IndexedList *contactsList();
 	Dialogs::IndexedList *dialogsList();
 	Dialogs::IndexedList *contactsNoDialogsList();
@@ -250,17 +236,18 @@ public:
 	void sendMessage(const MessageToSend &message);
 	void saveRecentHashtags(const QString &text);
 
-	void unreadCountChanged(History *history);
+	void unreadCountChanged(not_null<History*> history);
 
+	// While HistoryInner is not HistoryView::ListWidget.
 	TimeMs highlightStartTime(not_null<const HistoryItem*> item) const;
+	bool historyInSelectionMode() const;
 
 	void sendBotCommand(PeerData *peer, UserData *bot, const QString &cmd, MsgId replyTo);
 	void hideSingleUseKeyboard(PeerData *peer, MsgId replyTo);
 	bool insertBotCommand(const QString &cmd);
 
-	void jumpToDate(not_null<PeerData*> peer, const QDate &date);
-	void searchMessages(const QString &query, PeerData *inPeer);
-	void itemEdited(HistoryItem *item);
+	void searchMessages(const QString &query, Dialogs::Key inChat);
+	void itemEdited(not_null<HistoryItem*> item);
 
 	void checkLastUpdate(bool afterSleep);
 
@@ -282,33 +269,27 @@ public:
 	void messageDataReceived(ChannelData *channel, MsgId msgId);
 	void updateBotKeyboard(History *h);
 
-	void pushReplyReturn(HistoryItem *item);
+	void pushReplyReturn(not_null<HistoryItem*> item);
 
 	void cancelForwarding(not_null<History*> history);
 	void finishForwarding(not_null<History*> history);
 
-	void mediaMarkRead(not_null<DocumentData*> data);
-	void mediaMarkRead(const base::flat_set<not_null<HistoryItem*>> &items);
-	void mediaMarkRead(not_null<HistoryItem*> item);
-
-	void webPageUpdated(WebPageData *page);
-	void gameUpdated(GameData *game);
 	void updateMutedIn(TimeMs delay);
 
-	void choosePeer(PeerId peerId, MsgId showAtMsgId); // does offerPeer or showPeerHistory
+	// Does offerPeer or showPeerHistory.
+	void choosePeer(PeerId peerId, MsgId showAtMsgId);
 	void clearBotStartToken(PeerData *peer);
 
 	void ptsWaiterStartTimerFor(ChannelData *channel, int32 ms); // ms <= 0 - stop timer
 	void feedUpdates(const MTPUpdates &updates, uint64 randomId = 0);
-	void feedUpdate(const MTPUpdate &update);
 
 	void ctrlEnterSubmitUpdated();
 	void setInnerFocus();
 
 	void scheduleViewIncrement(HistoryItem *item);
 
-	void gotRangeDifference(ChannelData *channel, const MTPupdates_ChannelDifference &diff);
 	void onSelfParticipantUpdated(ChannelData *channel);
+	void feedChannelDifference(const MTPDupdates_channelDifference &data);
 
 	// Mayde public for ApiWrap, while it is still here.
 	// Better would be for this to be moved to ApiWrap.
@@ -324,6 +305,8 @@ public:
 
 	void documentLoadProgress(DocumentData *document);
 
+	void searchInChat(Dialogs::Key chat);
+
 	void app_sendBotCallback(
 		not_null<const HistoryMessageMarkupButton*> button,
 		not_null<const HistoryItem*> msg,
@@ -342,7 +325,6 @@ public:
 	void notify_inlineKeyboardMoved(const HistoryItem *item, int oldKeyboardTop, int newKeyboardTop);
 	bool notify_switchInlineBotButtonReceived(const QString &query, UserData *samePeerBot, MsgId samePeerReplyTo);
 	void notify_userIsBotChanged(UserData *bot);
-	void notify_userIsContactChanged(UserData *user, bool fromThisApp);
 	void notify_migrateUpdated(PeerData *peer);
 	void notify_historyMuteUpdated(History *history);
 
@@ -357,8 +339,6 @@ signals:
 	void dialogsUpdated();
 
 public slots:
-	void webPagesOrGamesUpdate();
-
 	void documentLoadProgress(FileLoader *loader);
 	void documentLoadFailed(FileLoader *loader, bool started);
 	void inlineResultLoadProgress(FileLoader *loader);
@@ -374,10 +354,6 @@ public slots:
 	void updateOnline(bool gotOtherOffline = false);
 	void checkIdleFinish();
 
-	void onHistoryShown(History *history, MsgId atMsgId);
-
-	void searchInPeer(PeerData *peer);
-
 	void onUpdateNotifySettings();
 
 	void onCacheBackground();
@@ -397,7 +373,12 @@ protected:
 private:
 	struct Float {
 		template <typename ToggleCallback, typename DraggedCallback>
-		Float(QWidget *parent, HistoryItem *item, ToggleCallback callback, DraggedCallback dragged);
+		Float(
+			QWidget *parent,
+			not_null<Window::Controller*> controller,
+			not_null<HistoryItem*> item,
+			ToggleCallback toggle,
+			DraggedCallback dragged);
 
 		bool hiddenByWidget = false;
 		bool hiddenByHistory = false;
@@ -437,12 +418,13 @@ private:
 	void updateMediaPlaylistPosition(int x);
 	void updateControlsGeometry();
 	void updateDialogsWidthAnimated();
-	void updateThirdColumnToCurrentPeer(
-		PeerData *peer,
+	void updateThirdColumnToCurrentChat(
+		Dialogs::Key key,
 		bool canWrite);
 	[[nodiscard]] bool saveThirdSectionToStackBack() const;
-	[[nodiscard]] auto thirdSectionForCurrentMainSection(
-		not_null<PeerData*> peer) -> std::unique_ptr<Window::SectionMemento>;
+	[[nodiscard]] auto thirdSectionForCurrentMainSection(Dialogs::Key key)
+		-> std::unique_ptr<Window::SectionMemento>;
+	void userIsContactUpdated(not_null<UserData*> user);
 
 	void createPlayer();
 	void switchToPanelPlayer();
@@ -458,7 +440,6 @@ private:
 	void messagesAffected(
 		not_null<PeerData*> peer,
 		const MTPmessages_AffectedMessages &result);
-	void messagesContentsRead(const MTPmessages_AffectedMessages &result);
 
 	Window::SectionSlideParams prepareShowAnimation(
 		bool willHaveTopBarShadow);
@@ -488,11 +469,15 @@ private:
 	bool failChannelDifference(ChannelData *channel, const RPCError &err);
 	void failDifferenceStartTimerFor(ChannelData *channel);
 
-	void feedUpdateVector(const MTPVector<MTPUpdate> &updates, bool skipMessageIds = false);
+	void feedUpdateVector(
+		const MTPVector<MTPUpdate> &updates,
+		bool skipMessageIds = false);
+	// Doesn't call sendHistoryChangeNotifications itself.
 	void feedMessageIds(const MTPVector<MTPUpdate> &updates);
+	// Doesn't call sendHistoryChangeNotifications itself.
+	void feedUpdate(const MTPUpdate &update);
 
 	void deleteHistoryPart(DeleteHistoryRequest request, const MTPmessages_AffectedHistory &result);
-	void deleteAllFromUserPart(DeleteAllFromUserParams params, const MTPmessages_AffectedHistory &result);
 
 	void updateReceived(const mtpPrime *from, const mtpPrime *end);
 	bool updateFail(const RPCError &e);
@@ -513,6 +498,7 @@ private:
 
 	void clearCachedBackground();
 	void checkCurrentFloatPlayer();
+	void createFloatPlayer(not_null<HistoryItem*> item);
 	void toggleFloatPlayer(not_null<Float*> instance);
 	void checkFloatPlayerVisibility();
 	void updateFloatPlayerPosition(not_null<Float*> instance);
@@ -554,10 +540,6 @@ private:
 	not_null<Window::Controller*> _controller;
 	bool _started = false;
 
-	OrderedSet<WebPageId> _webPagesUpdated;
-	OrderedSet<GameId> _gamesUpdated;
-	QTimer _webPageOrGameUpdater;
-
 	SingleTimer _updateMutedTimer;
 
 	QString _inviteHash;
@@ -593,8 +575,6 @@ private:
 	QPointer<ConfirmBox> _forwardConfirm; // for single column layout
 	object_ptr<HistoryHider> _hider = { nullptr };
 	std::vector<std::unique_ptr<StackItem>> _stack;
-	PeerData *_peerInStack = nullptr;
-	MsgId _msgIdInStack = 0;
 
 	int _playerHeight = 0;
 	int _callTopBarHeight = 0;
