@@ -1,31 +1,24 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "media/player/media_player_float.h"
 
 #include <rpl/merge.h>
 #include "data/data_document.h"
+#include "data/data_session.h"
+#include "data/data_media_types.h"
 #include "history/history_media.h"
+#include "history/history_item.h"
+#include "history/view/history_view_element.h"
 #include "media/media_clip_reader.h"
-#include "media/view/media_clip_playback.h"
 #include "media/media_audio.h"
+#include "media/view/media_clip_playback.h"
+#include "media/player/media_player_round_controller.h"
+#include "window/window_controller.h"
 #include "auth_session.h"
 #include "styles/style_media_player.h"
 #include "styles/style_history.h"
@@ -35,17 +28,19 @@ namespace Player {
 
 Float::Float(
 	QWidget *parent,
-	HistoryItem *item,
+	not_null<Window::Controller*> controller,
+	not_null<HistoryItem*> item,
 	base::lambda<void(bool visible)> toggleCallback,
 	base::lambda<void(bool closed)> draggedCallback)
 : RpWidget(parent)
+, _controller(controller)
 , _item(item)
 , _toggleCallback(std::move(toggleCallback))
 , _draggedCallback(std::move(draggedCallback)) {
-	auto media = _item->getMedia();
+	auto media = _item->media();
 	Assert(media != nullptr);
 
-	auto document = media->getDocument();
+	auto document = media->document();
 	Assert(document != nullptr);
 	Assert(document->isVideoMessage());
 
@@ -55,9 +50,7 @@ Float::Float(
 
 	prepareShadow();
 
-	rpl::merge(
-		Auth().data().itemLayoutChanged(),
-		Auth().data().itemRepaintRequest()
+	Auth().data().itemRepaintRequest(
 	) | rpl::start_with_next([this](auto item) {
 		if (_item == item) {
 			repaintItem();
@@ -109,10 +102,9 @@ float64 Float::outRatio() const {
 }
 
 void Float::mouseReleaseEvent(QMouseEvent *e) {
-	if (_down) {
-		_down = false;
-		if (auto media = _item ? _item->getMedia() : nullptr) {
-			media->playInline();
+	if (base::take(_down) && _item) {
+		if (const auto controller = _controller->roundVideo(_item)) {
+			controller->pauseResume();
 		}
 	}
 	if (_drag) {
@@ -130,8 +122,8 @@ void Float::finishDrag(bool closed) {
 void Float::mouseDoubleClickEvent(QMouseEvent *e) {
 	if (_item) {
 		// Handle second click.
-		if (auto media = _item->getMedia()) {
-			media->playInline();
+		if (const auto controller = _controller->roundVideo(_item)) {
+			controller->pauseResume();
 		}
 		Ui::showPeerHistoryAtItem(_item);
 	}
@@ -179,7 +171,8 @@ void Float::paintEvent(QPaintEvent *e) {
 	auto inner = getInnerRect();
 	p.drawImage(inner.topLeft(), _frame);
 
-	auto progress = _roundPlayback ? _roundPlayback->value(getms()) : 1.;
+	const auto playback = getPlayback();
+	const auto progress = playback ? playback->value(getms()) : 1.;
 	if (progress > 0.) {
 		auto pen = st::historyVideoMessageProgressFg->p;
 		auto was = p.pen();
@@ -202,9 +195,12 @@ void Float::paintEvent(QPaintEvent *e) {
 }
 
 Clip::Reader *Float::getReader() const {
-	if (auto media = _item ? _item->getMedia() : nullptr) {
-		if (auto reader = media->getClipReader()) {
-			if (reader->started() && reader->mode() == Clip::Reader::Mode::Video) {
+	if (detached()) {
+		return nullptr;
+	}
+	if (const auto controller = _controller->roundVideo(_item)) {
+		if (const auto reader = controller->reader()) {
+			if (reader->started()) {
 				return reader;
 			}
 		}
@@ -212,8 +208,18 @@ Clip::Reader *Float::getReader() const {
 	return nullptr;
 }
 
+Clip::Playback *Float::getPlayback() const {
+	if (detached()) {
+		return nullptr;
+	}
+	if (const auto controller = _controller->roundVideo(_item)) {
+		return controller->playback();
+	}
+	return nullptr;
+}
+
 bool Float::hasFrame() const {
-	if (auto reader = getReader()) {
+	if (const auto reader = getReader()) {
 		return !reader->current().isNull();
 	}
 	return false;
@@ -222,14 +228,15 @@ bool Float::hasFrame() const {
 bool Float::fillFrame() {
 	auto creating = _frame.isNull();
 	if (creating) {
-		_frame = QImage(getInnerRect().size() * cIntRetinaFactor(), QImage::Format_ARGB32_Premultiplied);
+		_frame = QImage(
+			getInnerRect().size() * cIntRetinaFactor(),
+			QImage::Format_ARGB32_Premultiplied);
 		_frame.setDevicePixelRatio(cRetinaFactor());
 	}
-	auto frameInner = [this] {
-		return QRect(0, 0, _frame.width() / cIntRetinaFactor(), _frame.height() / cIntRetinaFactor());
+	auto frameInner = [&] {
+		return QRect(QPoint(), _frame.size() / cIntRetinaFactor());
 	};
-	if (auto reader = getReader()) {
-		updatePlayback();
+	if (const auto reader = getReader()) {
 		auto frame = reader->current();
 		if (!frame.isNull()) {
 			_frame.fill(Qt::transparent);
@@ -252,21 +259,6 @@ bool Float::fillFrame() {
 	return false;
 }
 
-void Float::updatePlayback() {
-	if (_item) {
-		if (!_roundPlayback) {
-			_roundPlayback = std::make_unique<Media::Clip::Playback>();
-			_roundPlayback->setValueChangedCallback([this](float64 value) {
-				update();
-			});
-		}
-		auto state = Media::Player::mixer()->currentState(AudioMsgId::Type::Voice);
-		if (state.id.contextId() == _item->fullId()) {
-			_roundPlayback->updateState(state);
-		}
-	}
-}
-
 void Float::repaintItem() {
 	update();
 	if (hasFrame() && _toggleCallback) {
