@@ -1,27 +1,18 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
 #include "ui/rp_widget.h"
 
+namespace Window {
+class Controller;
+} // namespace Window
+
 namespace Media {
 namespace Clip {
 class Playback;
@@ -31,7 +22,12 @@ namespace Player {
 
 class Float : public Ui::RpWidget, private base::Subscriber {
 public:
-	Float(QWidget *parent, HistoryItem *item, base::lambda<void(bool visible)> toggleCallback, base::lambda<void(bool closed)> draggedCallback);
+	Float(
+		QWidget *parent,
+		not_null<Window::Controller*> controller,
+		not_null<HistoryItem*> item,
+		base::lambda<void(bool visible)> toggleCallback,
+		base::lambda<void(bool closed)> draggedCallback);
 
 	HistoryItem *item() const {
 		return _item;
@@ -72,14 +68,15 @@ protected:
 private:
 	float64 outRatio() const;
 	Clip::Reader *getReader() const;
+	Clip::Playback *getPlayback() const;
 	void repaintItem();
 	void prepareShadow();
 	bool hasFrame() const;
 	bool fillFrame();
 	QRect getInnerRect() const;
-	void updatePlayback();
 	void finishDrag(bool closed);
 
+	not_null<Window::Controller*> _controller;
 	HistoryItem *_item = nullptr;
 	base::lambda<void(bool visible)> _toggleCallback;
 
@@ -94,8 +91,6 @@ private:
 	QPoint _dragLocalPoint;
 	base::lambda<void(bool closed)> _draggedCallback;
 
-	std::unique_ptr<Clip::Playback> _roundPlayback;
-
 };
 
 } // namespace Player
