@@ -1,30 +1,19 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "media/player/media_player_panel.h"
 
 #include "media/player/media_player_cover.h"
 #include "media/player/media_player_instance.h"
 #include "info/media/info_media_list_widget.h"
-#include "history/history_media.h"
+#include "history/history.h"
+#include "history/history_item.h"
 #include "data/data_document.h"
+#include "data/data_media_types.h"
 #include "ui/widgets/shadow.h"
 #include "ui/widgets/scroll_area.h"
 #include "mainwindow.h"
@@ -269,8 +258,8 @@ void Panel::refreshList() {
 	const auto contextId = current.contextId();
 	const auto peer = [&]() -> PeerData* {
 		const auto item = contextId ? App::histItemById(contextId) : nullptr;
-		const auto media = item ? item->getMedia() : nullptr;
-		const auto document = media ? media->getDocument() : nullptr;
+		const auto media = item ? item->media() : nullptr;
+		const auto document = media ? media->document() : nullptr;
 		if (!document || !document->isSharedMediaMusic()) {
 			return nullptr;
 		}
@@ -366,8 +355,8 @@ void Panel::setCloseCallback(ButtonCallback &&callback) {
 	}
 }
 
-not_null<PeerData*> Panel::peer() const {
-	return _listPeer;
+Info::Key Panel::key() const {
+	return Info::Key(_listPeer);
 }
 
 PeerData *Panel::migrated() const {
