@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "media/view/media_view_group_thumbs.h"
 
@@ -24,7 +11,11 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "data/data_user_photos.h"
 #include "data/data_photo.h"
 #include "data/data_document.h"
+#include "data/data_media_types.h"
+#include "data/data_session.h"
+#include "history/history.h"
 #include "history/history_media.h"
+#include "auth_session.h"
 #include "styles/style_mediaview.h"
 
 namespace Media {
@@ -481,14 +472,14 @@ void GroupThumbs::animatePreviouslyAlive(
 
 auto GroupThumbs::createThumb(Key key) -> std::unique_ptr<Thumb> {
 	if (const auto photoId = base::get_if<PhotoId>(&key)) {
-		const auto photo = App::photo(*photoId);
+		const auto photo = Auth().data().photo(*photoId);
 		return createThumb(key, photo->date ? photo->thumb : ImagePtr());
 	} else if (const auto msgId = base::get_if<FullMsgId>(&key)) {
 		if (const auto item = App::histItemById(*msgId)) {
-			if (const auto media = item->getMedia()) {
-				if (const auto photo = media->getPhoto()) {
+			if (const auto media = item->media()) {
+				if (const auto photo = media->photo()) {
 					return createThumb(key, photo->thumb);
-				} else if (const auto document = media->getDocument()) {
+				} else if (const auto document = media->document()) {
 					return createThumb(key, document->thumb);
 				}
 			}
@@ -516,9 +507,7 @@ auto GroupThumbs::validateCacheEntry(Key key) -> not_null<Thumb*> {
 }
 
 void GroupThumbs::markCacheStale() {
-	while (!_dying.empty()) {
-		_dying.pop_back();
-	}
+	_dying.clear();
 	for (const auto &cacheItem : _cache) {
 		const auto &thumb = cacheItem.second;
 		thumb->setState(Thumb::State::Unknown);
