@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -54,20 +41,20 @@ enum class MTPDmessage_ClientFlag : uint32 {
 	// message is a group migrate (group -> supergroup) service message
 	f_is_group_migrate = (1U << 29),
 
-	// message needs initDimensions() + resize() + paint()
-	f_pending_init_dimensions = (1U << 28),
+	//// message needs initDimensions() + resize() + paint()
+	//f_pending_init_dimensions = (1U << 28),
 
-	// message needs resize() + paint()
-	f_pending_resize = (1U << 27),
+	//// message needs resize() + paint()
+	//f_pending_resize = (1U << 27),
 
-	// message needs paint()
-	f_pending_paint = (1U << 26),
+	//// message needs paint()
+	//f_pending_paint = (1U << 26),
 
-	// message is attached to previous one when displaying the history
-	f_attach_to_previous = (1U << 25),
+	//// message is attached to previous one when displaying the history
+	//f_attach_to_previous = (1U << 25),
 
-	// message is attached to next one when displaying the history
-	f_attach_to_next = (1U << 24),
+	//// message is attached to next one when displaying the history
+	//f_attach_to_next = (1U << 24),
 
 	// message was sent from inline bot, need to re-set media when sent
 	f_from_inline_bot = (1U << 23),
@@ -81,8 +68,8 @@ enum class MTPDmessage_ClientFlag : uint32 {
 	// message has an admin badge in supergroup
 	f_has_admin_badge = (1U << 20),
 
-	// message is not displayed because it is part of a group
-	f_hidden_by_group = (1U << 19),
+	//// message is not displayed because it is part of a group
+	//f_hidden_by_group = (1U << 19),
 
 	// update this when adding new client side flags
 	MIN_FIELD = (1U << 19),
