@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -25,10 +12,16 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "ui/effects/radial_animation.h"
 #include "styles/style_overview.h"
 
+class HistoryMedia;
+
 namespace style {
 struct RoundCheckbox;
 } // namespace style
 
+namespace Data {
+class Media;
+} // namespace Data
+
 namespace Overview {
 namespace Layout {
 
@@ -60,10 +53,7 @@ public:
 	virtual DocumentData *getDocument() const {
 		return nullptr;
 	}
-	MsgId msgId() const {
-		auto item = getItem();
-		return item ? item->id : 0;
-	}
+	MsgId msgId() const;
 
 	virtual void invalidateCache() {
 	}
@@ -74,6 +64,8 @@ class ItemBase : public AbstractItem {
 public:
 	ItemBase(not_null<HistoryItem*> parent);
 
+	QDateTime dateTime() const;
+
 	void setPosition(int position) {
 		_position = position;
 	}
@@ -113,7 +105,8 @@ private:
 	void ensureCheckboxCreated();
 
 	int _position = 0;
-	not_null<HistoryItem*> _parent;
+	const not_null<HistoryItem*> _parent;
+	const QDateTime _dateTime;
 	std::unique_ptr<Checkbox> _check;
 
 };
@@ -184,7 +177,7 @@ private:
 
 };
 
-struct Info : public RuntimeComponent<Info> {
+struct Info : public RuntimeComponent<Info, LayoutItemBase> {
 	int top = 0;
 };
 
@@ -210,9 +203,9 @@ public:
 	void initDimensions() override;
 	int32 resizeGetHeight(int32 width) override;
 	void paint(Painter &p, const QRect &clip, TextSelection selection, const PaintContext *context) override;
-	HistoryTextState getState(
+	TextState getState(
 		QPoint point,
-		HistoryStateRequest request) const override;
+		StateRequest request) const override;
 
 private:
 	not_null<PhotoData*> _data;
@@ -232,9 +225,9 @@ public:
 	void initDimensions() override;
 	int32 resizeGetHeight(int32 width) override;
 	void paint(Painter &p, const QRect &clip, TextSelection selection, const PaintContext *context) override;
-	HistoryTextState getState(
+	TextState getState(
 		QPoint point,
-		HistoryStateRequest request) const override;
+		StateRequest request) const override;
 
 protected:
 	float64 dataProgress() const override;
@@ -263,9 +256,9 @@ public:
 
 	void initDimensions() override;
 	void paint(Painter &p, const QRect &clip, TextSelection selection, const PaintContext *context) override;
-	HistoryTextState getState(
+	TextState getState(
 		QPoint point,
-		HistoryStateRequest request) const override;
+		StateRequest request) const override;
 
 protected:
 	float64 dataProgress() const override;
@@ -298,9 +291,9 @@ public:
 
 	void initDimensions() override;
 	void paint(Painter &p, const QRect &clip, TextSelection selection, const PaintContext *context) override;
-	HistoryTextState getState(
+	TextState getState(
 		QPoint point,
-		HistoryStateRequest request) const override;
+		StateRequest request) const override;
 
 	virtual DocumentData *getDocument() const override {
 		return _data;
@@ -337,14 +330,14 @@ class Link : public ItemBase {
 public:
 	Link(
 		not_null<HistoryItem*> parent,
-		HistoryMedia *media);
+		Data::Media *media);
 
 	void initDimensions() override;
 	int32 resizeGetHeight(int32 width) override;
 	void paint(Painter &p, const QRect &clip, TextSelection selection, const PaintContext *context) override;
-	HistoryTextState getState(
+	TextState getState(
 		QPoint point,
-		HistoryStateRequest request) const override;
+		StateRequest request) const override;
 
 protected:
 	const style::RoundCheckbox &checkboxStyle() const override;
