@@ -1,31 +1,17 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "platform/mac/notifications_manager_mac.h"
 
 #include "platform/platform_specific.h"
 #include "platform/mac/mac_utilities.h"
-#include "styles/style_window.h"
+#include "history/history.h"
 #include "mainwindow.h"
-#include "base/task_queue.h"
-#include "base/variant.h"
+#include "styles/style_window.h"
 
 #include <thread>
 #include <Cocoa/Cocoa.h>
@@ -87,7 +73,7 @@ NSImage *qt_mac_create_nsimage(const QPixmap &pm);
 	auto notificationManagerId = managerIdObject ? [managerIdObject unsignedLongLongValue] : 0ULL;
 	DEBUG_LOG(("Received notification with instance %1, mine: %2").arg(notificationManagerId).arg(_managerId));
 	if (notificationManagerId != _managerId) { // other app instance notification
-		base::TaskQueue::Main().Put([] {
+		crl::on_main([] {
 			// Usually we show and activate main window when the application
 			// is activated (receives applicationDidBecomeActive: notification).
 			//
@@ -112,17 +98,15 @@ NSImage *qt_mac_create_nsimage(const QPixmap &pm);
 	NSNumber *msgObject = [notificationUserInfo objectForKey:@"msgid"];
 	auto notificationMsgId = msgObject ? [msgObject intValue] : 0;
 	if (notification.activationType == NSUserNotificationActivationTypeReplied) {
-		auto notificationReply = QString::fromUtf8([[[notification response] string] UTF8String]);
-		base::TaskQueue::Main().Put([manager = _manager, notificationPeerId, notificationMsgId, notificationReply] {
-			if (manager) {
-				manager->notificationReplied(notificationPeerId, notificationMsgId, notificationReply);
-			}
+		const auto notificationReply = QString::fromUtf8([[[notification response] string] UTF8String]);
+		const auto manager = _manager;
+		crl::on_main(manager, [=] {
+			manager->notificationReplied(notificationPeerId, notificationMsgId, notificationReply);
 		});
 	} else if (notification.activationType == NSUserNotificationActivationTypeContentsClicked) {
-		base::TaskQueue::Main().Put([manager = _manager, notificationPeerId, notificationMsgId] {
-			if (manager) {
-				manager->notificationActivated(notificationPeerId, notificationMsgId);
-			}
+		const auto manager = _manager;
+		crl::on_main(manager, [=] {
+			manager->notificationActivated(notificationPeerId, notificationMsgId);
 		});
 	}
 
@@ -214,9 +198,9 @@ Manager::Private::Private(Manager *manager)
 	subscribe(Global::RefWorkMode(), [this](DBIWorkMode mode) {
 		// We need to update the delegate _after_ the tray icon change was done in Qt.
 		// Because Qt resets the delegate.
-		base::TaskQueue::Main().Put(base::lambda_guarded(this, [this] {
+		crl::on_main(this, [=] {
 			updateDelegate();
-		}));
+		});
 	});
 }
 
