@@ -43,16 +43,16 @@ GroupMembersWidget::GroupMembersWidget(
 		notifyPeerUpdated(update);
 	}));
 
-	setRemovedCallback([this, peer](PeerData *selectedPeer) {
+	setRemovedCallback([=](PeerData *selectedPeer) {
 		removePeer(selectedPeer);
 	});
-	setSelectedCallback([this](PeerData *selectedPeer) {
+	setSelectedCallback([=](PeerData *selectedPeer) {
 		Ui::showPeerProfile(selectedPeer);
 	});
-	setUpdateItemCallback([this](Item *item) {
+	setUpdateItemCallback([=](Item *item) {
 		updateItemStatusText(item);
 	});
-	setPreloadMoreCallback([this] {
+	setPreloadMoreCallback([=] {
 		preloadMore();
 	});
 
