@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "catch.hpp"
 
@@ -414,6 +401,35 @@ TEST_CASE("basic operators tests", "[rpl::operators]") {
 		REQUIRE(*sum == "0-11-22-3");
 	}
 
+	SECTION("combine_previous test") {
+		auto sum = std::make_shared<std::string>("");
+		{
+			rpl::lifetime lifetime;
+			event_stream<int> a;
+
+			a.events(
+			) | combine_previous(
+			) | start_with_next([=](int previous, int next) {
+				*sum += std::to_string(previous) + ' ';
+				*sum += std::to_string(next) + ' ';
+			}, lifetime);
+
+			a.events(
+			) | combine_previous(
+				5
+			) | start_with_next([=](int previous, int next) {
+				*sum += std::to_string(10 + previous) + ' ';
+				*sum += std::to_string(next) + ' ';
+			}, lifetime);
+
+			a.fire(1);
+			a.fire(2);
+			a.fire(3);
+			a.fire(4);
+		}
+		REQUIRE(*sum == "15 1 1 2 11 2 2 3 12 3 3 4 13 4 ");
+	}
+
 	SECTION("take test") {
 		auto sum = std::make_shared<std::string>("");
 		{
