@@ -1,26 +1,14 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "settings/settings_cover.h"
 
 #include "data/data_photo.h"
+#include "data/data_session.h"
 #include "ui/widgets/labels.h"
 #include "ui/widgets/buttons.h"
 #include "ui/special_buttons.h"
@@ -97,7 +85,7 @@ PhotoData *CoverWidget::validatePhoto() const {
 	Expects(_self != nullptr);
 
 	const auto photo = _self->userpicPhotoId()
-		? App::photo(_self->userpicPhotoId())
+		? Auth().data().photo(_self->userpicPhotoId()).get()
 		: nullptr;
 	_userpicButton->setPointerCursor(photo != nullptr && photo->date != 0);
 	if (_self->userpicPhotoUnknown() || (photo && !photo->date)) {
