@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "settings/settings_info_widget.h"
 
@@ -27,8 +14,10 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "boxes/username_box.h"
 #include "boxes/add_contact_box.h"
 #include "boxes/change_phone_box.h"
+#include "data/data_session.h"
 #include "observer_peer.h"
 #include "messenger.h"
+#include "auth_session.h"
 
 namespace Settings {
 
@@ -60,12 +49,8 @@ void InfoWidget::refreshControls() {
 
 void InfoWidget::refreshMobileNumber() {
 	TextWithEntities phoneText;
-	if (auto user = self()->asUser()) {
-		if (!user->phone().isEmpty()) {
-			phoneText.text = App::formatPhone(user->phone());
-		} else {
-			phoneText.text = App::phoneFromSharedContact(peerToUser(user->id));
-		}
+	if (const auto user = self()->asUser()) {
+		phoneText.text = Auth().data().findContactPhone(user);
 	}
 	setLabeledText(
 		_mobileNumber,
