@@ -324,6 +324,10 @@ protected:
 		return _data.height();
 	}
 
+	virtual bool hasLocalCopy() const {
+		return false;
+	}
+
 	mutable QByteArray _saved, _format;
 	mutable bool _forgot;
 	mutable QPixmap _data;
@@ -413,6 +417,8 @@ protected:
 	void setInformation(int32 size, int32 width, int32 height) override;
 	FileLoader *createLoader(LoadFromCloudSetting fromCloud, bool autoLoading) override;
 
+	bool hasLocalCopy() const override;
+
 	StorageImageLocation _location;
 	int32 _size;
 
@@ -434,6 +440,8 @@ protected:
 		return _box;
 	}
 
+	bool hasLocalCopy() const override;
+
 	WebFileLocation _location;
 	QSize _box;
 	int _width = 0;
@@ -496,6 +504,8 @@ protected:
 	int32 countWidth() const override;
 	int32 countHeight() const override;
 
+	bool hasLocalCopy() const override;
+
 private:
 	QString _url;
 	QSize _box;
