@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "ui/text/text_entity.h"
 
@@ -44,8 +31,8 @@ QString ExpressionMailNameAtEnd() {
 }
 
 QString ExpressionSeparators(const QString &additional) {
-	// UTF8 quotes
-	const auto quotes = QString::fromUtf8("\xC2\xAB\xC2\xBB\xE2\x80\x9C\xE2\x80\x9D\xE2\x80\x98\xE2\x80\x99");
+	// UTF8 quotes and ellipsis
+	const auto quotes = QString::fromUtf8("\xC2\xAB\xC2\xBB\xE2\x80\x9C\xE2\x80\x9D\xE2\x80\x98\xE2\x80\x99\xE2\x80\xA6");
 	return qsl("\\s\\.,:;<>|'\"\\[\\]\\{\\}\\~\\!\\?\\%\\^\\(\\)\\-\\+=\\x10") + quotes + additional;
 }
 
@@ -53,6 +40,10 @@ QString ExpressionHashtag() {
 	return qsl("(^|[") + ExpressionSeparators(qsl("`\\*/")) + qsl("])#[\\w]{2,64}([\\W]|$)");
 }
 
+QString ExpressionHashtagExclude() {
+	return qsl("^#?\\d+$");
+}
+
 QString ExpressionMention() {
 	return qsl("(^|[") + ExpressionSeparators(qsl("`\\*/")) + qsl("])@[A-Za-z_0-9]{1,32}([\\W]|$)");
 }
@@ -1154,6 +1145,11 @@ const QRegularExpression &RegExpHashtag() {
 	return result;
 }
 
+const QRegularExpression &RegExpHashtagExclude() {
+	static const auto result = CreateRegExp(ExpressionHashtagExclude());
+	return result;
+}
+
 const QRegularExpression &RegExpMention() {
 	static const auto result = CreateRegExp(ExpressionMention());
 	return result;
@@ -1462,6 +1458,8 @@ EntitiesInText EntitiesFromMTP(const QVector<MTPMessageEntity> &entities) {
 			case mtpc_messageEntityTextUrl: { auto &d = entity.c_messageEntityTextUrl(); result.push_back(EntityInText(EntityInTextCustomUrl, d.voffset.v, d.vlength.v, Clean(qs(d.vurl)))); } break;
 			case mtpc_messageEntityEmail: { auto &d = entity.c_messageEntityEmail(); result.push_back(EntityInText(EntityInTextEmail, d.voffset.v, d.vlength.v)); } break;
 			case mtpc_messageEntityHashtag: { auto &d = entity.c_messageEntityHashtag(); result.push_back(EntityInText(EntityInTextHashtag, d.voffset.v, d.vlength.v)); } break;
+			case mtpc_messageEntityCashtag: { auto &d = entity.c_messageEntityCashtag(); result.push_back(EntityInText(EntityInTextCashtag, d.voffset.v, d.vlength.v)); } break;
+			case mtpc_messageEntityPhone: break; // Skipping phones.
 			case mtpc_messageEntityMention: { auto &d = entity.c_messageEntityMention(); result.push_back(EntityInText(EntityInTextMention, d.voffset.v, d.vlength.v)); } break;
 			case mtpc_messageEntityMentionName: {
 				auto &d = entity.c_messageEntityMentionName();
@@ -1522,6 +1520,7 @@ MTPVector<MTPMessageEntity> EntitiesToMTP(const EntitiesInText &entities, Conver
 		case EntityInTextCustomUrl: v.push_back(MTP_messageEntityTextUrl(offset, length, MTP_string(entity.data()))); break;
 		case EntityInTextEmail: v.push_back(MTP_messageEntityEmail(offset, length)); break;
 		case EntityInTextHashtag: v.push_back(MTP_messageEntityHashtag(offset, length)); break;
+		case EntityInTextCashtag: v.push_back(MTP_messageEntityCashtag(offset, length)); break;
 		case EntityInTextMention: v.push_back(MTP_messageEntityMention(offset, length)); break;
 		case EntityInTextMentionName: {
 			auto inputUser = ([](const QString &data) -> MTPInputUser {
@@ -1793,6 +1792,13 @@ void ParseMarkdown(
 	}
 }
 
+TextWithEntities ParseEntities(const QString &text, int32 flags) {
+	const auto rich = ((flags & TextParseRichText) != 0);
+	auto result = TextWithEntities{ text, EntitiesInText() };
+	ParseEntities(result, flags, rich);
+	return result;
+}
+
 // Some code is duplicated in flattextarea.cpp!
 void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 	if (flags & TextParseMarkdown) { // parse markdown entities (bold, italic, code and pre)
@@ -1801,6 +1807,8 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 		ParseMarkdown(result, copy.entities, rich);
 	}
 
+	constexpr auto kNotFound = std::numeric_limits<int>::max();
+
 	auto newEntities = EntitiesInText();
 	bool withHashtags = (flags & TextParseHashtags);
 	bool withMentions = (flags & TextParseMentions);
@@ -1830,16 +1838,19 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 
 		EntityInTextType lnkType = EntityInTextUrl;
 		int32 lnkStart = 0, lnkLength = 0;
-		int32 domainStart = mDomain.hasMatch() ? mDomain.capturedStart() : INT_MAX,
-			domainEnd = mDomain.hasMatch() ? mDomain.capturedEnd() : INT_MAX,
-			explicitDomainStart = mExplicitDomain.hasMatch() ? mExplicitDomain.capturedStart() : INT_MAX,
-			explicitDomainEnd = mExplicitDomain.hasMatch() ? mExplicitDomain.capturedEnd() : INT_MAX,
-			hashtagStart = mHashtag.hasMatch() ? mHashtag.capturedStart() : INT_MAX,
-			hashtagEnd = mHashtag.hasMatch() ? mHashtag.capturedEnd() : INT_MAX,
-			mentionStart = mMention.hasMatch() ? mMention.capturedStart() : INT_MAX,
-			mentionEnd = mMention.hasMatch() ? mMention.capturedEnd() : INT_MAX,
-			botCommandStart = mBotCommand.hasMatch() ? mBotCommand.capturedStart() : INT_MAX,
-			botCommandEnd = mBotCommand.hasMatch() ? mBotCommand.capturedEnd() : INT_MAX;
+		auto domainStart = mDomain.hasMatch() ? mDomain.capturedStart() : kNotFound,
+			domainEnd = mDomain.hasMatch() ? mDomain.capturedEnd() : kNotFound,
+			explicitDomainStart = mExplicitDomain.hasMatch() ? mExplicitDomain.capturedStart() : kNotFound,
+			explicitDomainEnd = mExplicitDomain.hasMatch() ? mExplicitDomain.capturedEnd() : kNotFound,
+			hashtagStart = mHashtag.hasMatch() ? mHashtag.capturedStart() : kNotFound,
+			hashtagEnd = mHashtag.hasMatch() ? mHashtag.capturedEnd() : kNotFound,
+			mentionStart = mMention.hasMatch() ? mMention.capturedStart() : kNotFound,
+			mentionEnd = mMention.hasMatch() ? mMention.capturedEnd() : kNotFound,
+			botCommandStart = mBotCommand.hasMatch() ? mBotCommand.capturedStart() : kNotFound,
+			botCommandEnd = mBotCommand.hasMatch() ? mBotCommand.capturedEnd() : kNotFound;
+		auto hashtagIgnore = false;
+		auto mentionIgnore = false;
+
 		if (mHashtag.hasMatch()) {
 			if (!mHashtag.capturedRef(1).isEmpty()) {
 				++hashtagStart;
@@ -1847,6 +1858,12 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 			if (!mHashtag.capturedRef(2).isEmpty()) {
 				--hashtagEnd;
 			}
+			if (RegExpHashtagExclude().match(
+				result.text.mid(
+					hashtagStart + 1,
+					hashtagEnd - hashtagStart - 1)).hasMatch()) {
+				hashtagIgnore = true;
+			}
 		}
 		while (mMention.hasMatch()) {
 			if (!mMention.capturedRef(1).isEmpty()) {
@@ -1862,8 +1879,7 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 					mentionStart = mMention.capturedStart();
 					mentionEnd = mMention.capturedEnd();
 				} else {
-					mentionStart = INT_MAX;
-					mentionEnd = INT_MAX;
+					mentionIgnore = true;
 				}
 			} else {
 				break;
@@ -1877,7 +1893,11 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 				--botCommandEnd;
 			}
 		}
-		if (!mDomain.hasMatch() && !mExplicitDomain.hasMatch() && !mHashtag.hasMatch() && !mMention.hasMatch() && !mBotCommand.hasMatch()) {
+		if (!mDomain.hasMatch()
+			&& !mExplicitDomain.hasMatch()
+			&& !mHashtag.hasMatch()
+			&& !mMention.hasMatch()
+			&& !mBotCommand.hasMatch()) {
 			break;
 		}
 
@@ -1886,8 +1906,20 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 			domainEnd = explicitDomainEnd;
 			mDomain = mExplicitDomain;
 		}
-		if (mentionStart < hashtagStart && mentionStart < domainStart && mentionStart < botCommandStart) {
-			bool inCommand = checkTagStartInCommand(start, len, mentionStart, commandOffset, commandIsLink, inLink);
+		if (mentionStart < hashtagStart
+			&& mentionStart < domainStart
+			&& mentionStart < botCommandStart) {
+			if (mentionIgnore) {
+				offset = matchOffset = mentionEnd;
+				continue;
+			}
+			const auto inCommand = checkTagStartInCommand(
+				start,
+				len,
+				mentionStart,
+				commandOffset,
+				commandIsLink,
+				inLink);
 			if (inCommand || inLink) {
 				offset = matchOffset = commandOffset;
 				continue;
@@ -1896,8 +1928,19 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 			lnkType = EntityInTextMention;
 			lnkStart = mentionStart;
 			lnkLength = mentionEnd - mentionStart;
-		} else if (hashtagStart < domainStart && hashtagStart < botCommandStart) {
-			bool inCommand = checkTagStartInCommand(start, len, hashtagStart, commandOffset, commandIsLink, inLink);
+		} else if (hashtagStart < domainStart
+			&& hashtagStart < botCommandStart) {
+			if (hashtagIgnore) {
+				offset = matchOffset = hashtagEnd;
+				continue;
+			}
+			const auto inCommand = checkTagStartInCommand(
+				start,
+				len,
+				hashtagStart,
+				commandOffset,
+				commandIsLink,
+				inLink);
 			if (inCommand || inLink) {
 				offset = matchOffset = commandOffset;
 				continue;
@@ -1907,7 +1950,13 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 			lnkStart = hashtagStart;
 			lnkLength = hashtagEnd - hashtagStart;
 		} else if (botCommandStart < domainStart) {
-			bool inCommand = checkTagStartInCommand(start, len, botCommandStart, commandOffset, commandIsLink, inLink);
+			const auto inCommand = checkTagStartInCommand(
+				start,
+				len,
+				botCommandStart,
+				commandOffset,
+				commandIsLink,
+				inLink);
 			if (inCommand || inLink) {
 				offset = matchOffset = commandOffset;
 				continue;
@@ -1917,7 +1966,13 @@ void ParseEntities(TextWithEntities &result, int32 flags, bool rich) {
 			lnkStart = botCommandStart;
 			lnkLength = botCommandEnd - botCommandStart;
 		} else {
-			auto inCommand = checkTagStartInCommand(start, len, domainStart, commandOffset, commandIsLink, inLink);
+			const auto inCommand = checkTagStartInCommand(
+				start,
+				len,
+				domainStart,
+				commandOffset,
+				commandIsLink,
+				inLink);
 			if (inCommand || inLink) {
 				offset = matchOffset = commandOffset;
 				continue;
