@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
@@ -455,6 +442,16 @@ QPointer<const Widget> make_weak(const Widget *object) {
 	return QPointer<const Widget>(object);
 }
 
+template <typename Widget>
+QPointer<Widget> make_weak(not_null<Widget*> object) {
+	return QPointer<Widget>(object.get());
+}
+
+template <typename Widget>
+QPointer<const Widget> make_weak(not_null<const Widget*> object) {
+	return QPointer<const Widget>(object.get());
+}
+
 class SingleQueuedInvokation : public QObject {
 public:
 	SingleQueuedInvokation(base::lambda<void()> callback) : _callback(callback) {
