@@ -1,28 +1,16 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "window/themes/window_theme_editor.h"
 
 #include "window/themes/window_theme.h"
 #include "window/themes/window_theme_editor_block.h"
 #include "mainwindow.h"
+#include "layout.h"
 #include "storage/localstorage.h"
 #include "boxes/confirm_box.h"
 #include "styles/style_window.h"
@@ -35,7 +23,6 @@ Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
 #include "ui/widgets/checkbox.h"
 #include "ui/widgets/multi_select.h"
 #include "base/parse_helper.h"
-#include "base/task_queue.h"
 #include "base/zlib_help.h"
 #include "ui/toast/toast.h"
 #include "core/file_utilities.h"
@@ -718,7 +705,9 @@ Editor::Editor(QWidget*, const QString &path)
 
 		// This could be from inner->_context observable notification.
 		// We should not destroy it while iterating in subscribers.
-		base::TaskQueue::Main().Put(base::lambda_guarded(this, [this] { closeEditor(); }));
+		crl::on_main(this, [=] {
+			closeEditor();
+		});
 	});
 	_inner->setFocusCallback([this] {
 		App::CallDelayed(2 * st::boxDuration, this, [this] { _select->setInnerFocus(); });
