@@ -127,14 +127,14 @@ void EditorBlock::Row::setValue(QColor value) {
 }
 
 void EditorBlock::Row::fillValueString() {
-	auto addHex = [this](int code) {
+	auto addHex = [=](int code) {
 		if (code >= 0 && code < 10) {
 			_valueString.append('0' + code);
 		} else if (code >= 10 && code < 16) {
 			_valueString.append('a' + (code - 10));
 		}
 	};
-	auto addCode = [this, addHex](int code) {
+	auto addCode = [=](int code) {
 		addHex(code / 16);
 		addHex(code % 16);
 	};
@@ -459,7 +459,7 @@ void EditorBlock::enumerateRowsFrom(int top, Callback callback) const {
 int EditorBlock::resizeGetHeight(int newWidth) {
 	auto result = 0;
 	auto descriptionWidth = newWidth - st::themeEditorMargin.left() - st::themeEditorMargin.right();
-	enumerateRows([this, &result, descriptionWidth](Row &row) {
+	enumerateRows([&](Row &row) {
 		row.setTop(result);
 
 		auto height = row.height();
