@@ -1,22 +1,9 @@
 /*
 This file is part of Telegram Desktop,
-the official desktop version of Telegram messaging app, see https://telegram.org
+the official desktop application for the Telegram messaging service.
 
-Telegram Desktop is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-It is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-GNU General Public License for more details.
-
-In addition, as a special exception, the copyright holders give permission
-to link the code of portions of this program with the OpenSSL library.
-
-Full license: https://github.com/telegramdesktop/tdesktop/blob/master/LICENSE
-Copyright (c) 2014-2017 John Preston, https://desktop.telegram.org
+For license and copyright information please follow this link:
+https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "window/themes/window_theme_preview.h"
 
@@ -94,9 +81,9 @@ QString fillLetters(const QString &name) {
 
 class Generator {
 public:
-	Generator(const Instance &theme, const CurrentData &current);
+	Generator(const Instance &theme, CurrentData &&current);
 
-	QPixmap generate();
+	QImage generate();
 
 private:
 	enum class Status {
@@ -173,7 +160,7 @@ private:
 
 	const Instance &_theme;
 	const style::palette &_palette;
-	const CurrentData &_current;
+	CurrentData _current;
 	Painter *_p = nullptr;
 
 	QRect _rect;
@@ -350,16 +337,18 @@ void Generator::generateData() {
 	_bubbles.back().replyText.setText(st::messageTextStyle, "Mark Twain said that " + QString() + QChar(9757) + QChar(55356) + QChar(57339), Ui::DialogTextOptions());
 }
 
-Generator::Generator(const Instance &theme, const CurrentData &current)
+Generator::Generator(const Instance &theme, CurrentData &&current)
 : _theme(theme)
 , _palette(_theme.palette)
-, _current(current) {
+, _current(std::move(current)) {
 }
 
-QPixmap Generator::generate() {
+QImage Generator::generate() {
 	prepare();
 
-	auto result = QImage(_rect.size() * cIntRetinaFactor(), QImage::Format_ARGB32_Premultiplied);
+	auto result = QImage(
+		_rect.size() * cIntRetinaFactor(),
+		QImage::Format_ARGB32_Premultiplied);
 	result.setDevicePixelRatio(cRetinaFactor());
 	result.fill(st::themePreviewBg->c);
 
@@ -379,7 +368,7 @@ QPixmap Generator::generate() {
 	}
 	Platform::PreviewWindowFramePaint(result, _palette, _body, _rect.width());
 
-	return App::pixmapFromImageInPlace(std::move(result));
+	return result;
 }
 
 void Generator::paintHistoryList() {
@@ -408,11 +397,12 @@ void Generator::paintHistoryBackground() {
 			background.load(qsl(":/gui/art/bg.jpg"));
 			tiled = false;
 		} else {
-			background = _current.backgroundImage.toImage();
+			background = std::move(_current.backgroundImage);
 			tiled = _current.backgroundTiled;
 		}
 	}
-	background = std::move(background).convertToFormat(QImage::Format_ARGB32_Premultiplied);
+	background = std::move(background).convertToFormat(
+		QImage::Format_ARGB32_Premultiplied);
 	background.setDevicePixelRatio(cRetinaFactor());
 	_p->setClipRect(_history);
 	if (tiled) {
@@ -420,7 +410,10 @@ void Generator::paintHistoryBackground() {
 		auto height = background.height();
 		auto repeatTimesX = qCeil(_history.width() * cIntRetinaFactor() / float64(width));
 		auto repeatTimesY = qCeil((_history.height() - fromy) * cIntRetinaFactor() / float64(height));
-		auto imageForTiled = QImage(width * repeatTimesX, height * repeatTimesY, QImage::Format_ARGB32_Premultiplied);
+		auto imageForTiled = QImage(
+			width * repeatTimesX,
+			height * repeatTimesY,
+			QImage::Format_ARGB32_Premultiplied);
 		imageForTiled.setDevicePixelRatio(background.devicePixelRatio());
 		auto imageForTiledBytes = imageForTiled.bits();
 		auto bytesInLine = width * sizeof(uint32);
@@ -432,7 +425,8 @@ void Generator::paintHistoryBackground() {
 					imageForTiledBytes += bytesInLine;
 				}
 				imageBytes += background.bytesPerLine();
-				imageForTiledBytes += imageForTiled.bytesPerLine() - (repeatTimesX * bytesInLine);
+				imageForTiledBytes += imageForTiled.bytesPerLine()
+					- (repeatTimesX * bytesInLine);
 			}
 		}
 		_p->drawImage(_history.x(), _history.y() + fromy, imageForTiled);
@@ -891,13 +885,18 @@ void Generator::restoreTextPalette() {
 
 } // namespace
 
-std::unique_ptr<Preview> GeneratePreview(const QString &filepath, const CurrentData &data) {
+std::unique_ptr<Preview> GeneratePreview(
+		const QString &filepath,
+		CurrentData &&data) {
 	auto result = std::make_unique<Preview>();
 	result->path = filepath;
 	if (!LoadFromFile(filepath, &result->instance, &result->content)) {
 		return nullptr;
 	}
-	result->preview = Generator(result->instance, data).generate();
+	result->preview = Generator(
+		result->instance,
+		std::move(data)
+	).generate();
 	return result;
 }
 
